from typing import Any

from java.lang import Class, String


class TypeBuilder(object):
    def getBase(self):
        # type: () -> Class
        raise NotImplementedError

    def getDict(self, var1):
        # type: (Any) -> Any
        raise NotImplementedError

    def getDoc(self):
        # type: () -> String
        raise NotImplementedError

    def getIsBaseType(self):
        # type: () -> bool
        raise NotImplementedError

    def getName(self):
        # type: () -> String
        raise NotImplementedError

    def getTypeClass(self):
        # type: () -> Class
        raise NotImplementedError



"""SECS/GEM Functions.

The following functions allow you to interact with equipment defined by
the SECS/GEM module. Note that the SECS/GEM module must be installed
before these functions will be accessible.
"""

from __future__ import print_function

__all__ = [
    "copyEquipment",
    "deleteToolProgram",
    "enableDisableEquipment",
    "getResponse",
    "getToolProgram",
    "getToolProgramDataset",
    "sendRequest",
    "sendResponse",
    "startSimEventRun",
    "toDataSet",
    "toTreeDataSet",
]

from typing import Any, Dict, List, Optional, Tuple

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import String
from java.util import Date


def copyEquipment(
    equipmentSource,  # type: String
    newEquipmentName,  # type: String
    enabled,  # type: bool
    activeAddress,  # type: String
    activePort,  # type: int
    passiveAddress,  # type: String
    passivePort,  # type: int
    deviceId,  # type: int
    dbTablePrefix=None,  # type: Optional[String]
    description=None,  # type: Optional[String]
):
    # type: (...) -> None
    """Creates a copy of an equipment connection.

    Common settings can be overridden for the new connection.

    An exception is thrown if the new Equipment Connection cannot be
    created.

    Args:
        equipmentSource: Some new equipment settings will be retrieved
            from this equipment connection. Specify the source equipment
            connection name.
        newEquipmentName: The name of the new equipment connection.
        enabled: If set to False, the new equipment connection will be
            disabled after it is created.
        activeAddress: IP Address of new equipment. Must be specified if
            the SECS/GEM module is used in Active mode. Otherwise, do
            not use this parameter.
        activePort: Port number of new equipment. Must be specified if
            the SECS/GEM module is used in Active mode. Otherwise, do
            not use this parameter.
        passiveAddress: IP Address of new equipment. Must be specified
            if the SECS/GEM module is used in Passive mode. Otherwise,
            do not use this parameter.
        passivePort: Port number of new equipment. Must be specified if
            the SECS/GEM module is used in Passive mode. Otherwise, do
            not use this parameter.
        deviceId: Unique identifier of new equipment. This value must be
            an integer, and is specified within the equipment.
        dbTablePrefix: SECS/GEM database table names will use the
            specified prefix for the new equipment connection. If no
            prefix is specified, the description of the source equipment
            will be used. Optional.
        description: The description for the new equipment connection.
            If no description is specified, the description of the
            source equipment will be used. Optional.
    """
    print(
        equipmentSource,
        newEquipmentName,
        enabled,
        activeAddress,
        activePort,
        passiveAddress,
        passivePort,
        deviceId,
        dbTablePrefix,
        description,
    )


def deleteToolProgram(ppid):
    # type: (String) -> None
    """Deletes a process program from the Gateway.

    Args:
        ppid: The PPID that was sent from the tool when the S7F3 message
            was saved.
    """
    print(ppid)


def enableDisableEquipment(enable, names):
    # type: (bool, Tuple[String, ...]) -> List[unicode]
    """Enables or disables a Tuple of equipment connections from a
    script.

    Args:
        enable: Set to True to enable equipment connections, or set to
            False to disable them.
        names: A Tuple of Strings. Each String should match an Equipment
            Connection configured on the Gateway. If this parameter
            contains the name of an Equipment Connection that does not
            exist, then a message will be included in the List returned
            by this function.

    Returns:
         A List of unicode strings. Each string contains a message about
         an equipment connection that could not be enabled/disabled. If
         the list is empty, then all specified equipment connections
         were successfully modified.
    """
    print(enable)
    for name in names:
        print(name)
    return [unicode("")]


def getResponse(transactionID, equipment, timeout=5, poll=150):
    # type: (int, String, Optional[int], Optional[int]) -> Any
    """Attempts to retrieve a response message from the Gateway.

    The transaction id from the sent message is used to retrieve the
    response.

    Args:
        transactionID: The transactionID of the request and response.
            The transactionID is used to retrieve the response.
            Typically used in conjunction with
            system.secsgem.sendRequest to generate a transactionID.
        equipment: Name of equipment connection.
        timeout: Specifies in seconds how long to wait for a response
            before returning None. If omitted the timeout will be 5
            seconds. Optional.
        poll: Specifies in milliseconds how often to poll the system for
            a response. If omitted the poll will be 150 milliseconds.
            Optional.

    Returns:
        A Python object, typically a dictionary. The actual result is a
        JSON string that's decoded into a python object, as shown on the
        mapping on the system.util.jsonDecode page.
    """
    print(transactionID, equipment, timeout, poll)
    return ""


def getToolProgram(ppid):
    # type: (String) -> Dict[String, Any]
    """Returns a process program from the Gateway that was previously
    sent by a a tool in an S7F3 message.

    Args:
        ppid: The PPID that was sent from the tool when the S7F3 message
            was saved.

    Returns:
        A Python Dictionary containing the following keys: editDate,
        ppbody, bodyFormat.
    """
    print(ppid)
    return {
        "editDate": Date(),
        "ppBody": "program",
        "bodyFormat": "A",
    }


def getToolProgramDataset():
    # type: () -> BasicDataset
    """Returns a Dataset containing information about all stored process
    programs.

    Returns:
         A Dataset containing information about all stored process
         programs. Includes the following columns in order: ppid,
         editDate, bodyFormat.
    """
    return BasicDataset()


def sendRequest(streamFunction, reply, body, equipment):
    # type: (String, bool, Any, String) -> int
    """Sends a JSON-formatted SECS message to a tool.

    An equipment connection must be configured for the tool in the
    Gateway.

    Args:
        streamFunction: The stream and function of the SECS message to
            send. Example: "S1F13"
        reply: Whether or not the SECS message expects a reply message.
        body: This contains the body of a SECS message. The argument can
            be a Python Object or JSON string representing the body of a
            SECS message. If this argument is a string then it will be
            converted to a Python Object using the
            system.util.jsonDecode function.
        equipment: Name of the equipment connection to use.

    Returns:
        The transactionID of the SECS message response.
    """
    print(streamFunction, reply, body, equipment)
    return 0


def sendResponse(transactionID, systemBytes, streamFunction, body, equipment):
    # type: (int, int, String, Any, String) -> None
    """Sends a JSON-formatted SECS response message to a message sent by
    a tool.

    An equipment connection must be configured for the tool in the
    Gateway, and this must be used within a Message Handler to create a
    Custom Message Response Handler.

    Args:
        transactionID: The TxID of the response. The TxID from the
            received request in the payload of the message handler must
            be specified here.
        systemBytes: The SystemBytes of the response. The SystemBytes
            from the received request in the payload of the message
            handler must be specified here.
        streamFunction: The stream and function of the SECS message to
            send. Example: "S1F14".
        body: This contains the body of a SECS response. The argument
            can be a Python Object or JSON string representing the body
            of a SECS message. If this argument is a string then it will
            be converted to a Python Object using the
            system.util.jsonDecode function.
        equipment: Name of the equipment connection to use.
    """
    print(transactionID, systemBytes, streamFunction, body, equipment)


def startSimEventRun(simulatorName, eventRunName):
    # type: (String, String) -> None
    """Starts a configured simulator event run in the Gateway.

    Note, that this function only works with the simulators that come
    included with the SECS/GEM module.

    The function will throw an exception if the specified Event Run
    cannot be started.

    Args:
        simulatorName: The simulator that holds the configured event
            run. Will throw an exception if the specified simulator
            can't be found.
        eventRunName: The event run to start. Will throw an exception if
            the specified simulator can't be found.
    """
    print(simulatorName, eventRunName)


def toDataSet(secsObject):
    # type: (Any) -> BasicDataset
    """Converts a SECS message data structure, as returned by the
    system.secsgem.getResponse function, into a dataset and returns it.

    Args:
        secsObject: A Python object, such as Sequence or a Dictionary,
            representing a SECS message to convert to a dataset.

    Returns:
         A Dataset representing a SECS message.
    """
    print(secsObject)
    return BasicDataset()


def toTreeDataSet(dataset):
    # type: (BasicDataset) -> BasicDataset
    """Changes an existing dataset, as returned by the
    system.secsgem.toDataSet function, to make it usable for the Tree
    View component.

    Args:
        dataset: A dataset containing a SECS message. Note that this
            parameter cannot take a JSON message, so the object returned
            by system.secsgem.getResponse must first be processed by
            system.secsgem.toDataSet.

    Returns:
        A dataset containing a SECS message with the following columns,
        as suited for Vision's tree view component: "path", "text",
        "icon", "background", "foreground", "tooltip", "border",
        "selectedText", "selectedIcon", "selectedBackground",
        "selectedForeground", "selectedTooltip", "selectedBorder".
    """
    print(dataset)
    return BasicDataset()



from __future__ import print_function

__all__ = [
    "DimensionMismatchException",
    "MathIllegalArgumentException",
    "MathIllegalNumberException",
]

from typing import Any

from java.lang import IllegalArgumentException, String


class MathIllegalArgumentException(IllegalArgumentException):
    """Base class for all preconditions violation exceptions. In most
    cases, this class should not be instantiated directly: it should
    serve as a base class to create all the exceptions that have the
    semantics of the standard IllegalArgumentException.
    """

    def __init__(self, pattern, *args):
        # type: (String, *Any) -> None
        print(pattern, args)
        super(MathIllegalArgumentException, self).__init__()

    def getContext(self):
        pass


class MathIllegalNumberException(MathIllegalArgumentException):
    """Base class for exceptions raised by a wrong number.

    This class is not intended to be instantiated directly: it should
    serve as a base class to create all the exceptions that are raised
    because some precondition is violated by a number argument.
    """

    INTEGER_ZERO = 0

    def getArgument(self):
        pass


class DimensionMismatchException(MathIllegalNumberException):
    """Exception to be thrown when two dimensions differ."""

    def __init__(self, *args):
        # type: (*Any) -> None
        super(DimensionMismatchException, self).__init__("", *args)

    def getDimension(self):
        pass



__all__ = ["ToStringStyle"]

from typing import Any

from java.lang import Object, String, StringBuffer


class ToStringStyle(Object):
    DEFAULT_STYLE = None  # type: ToStringStyle
    JSON_STYLE = None  # type: ToStringStyle
    MULTI_LINE_STYLE = None  # type: ToStringStyle
    NO_CLASS_NAME_STYLE = None  # type: ToStringStyle
    NO_FIELD_NAMES_STYLE = None  # type: ToStringStyle
    SHORT_PREFIX_STYLE = None  # type: ToStringStyle
    SIMPLE_STYLE = None  # type: ToStringStyle

    def append(self, buffer, fieldName, *args):
        # type: (StringBuffer, String, Any) -> None
        pass

    def appendEnd(self, buffer, obj):
        # type: (StringBuffer, Any) -> None
        pass

    def appendStart(self, buffer, obj):
        # type: (StringBuffer, Any) -> None
        pass

    def appendSuper(self, buffer, superToString):
        # type: (StringBuffer, String) -> None
        pass

    def appendToString(self, buffer, toString):
        # type: (StringBuffer, String) -> None
        pass



__all__ = ["Request", "RequestWatcher"]

from java.lang import Exception as JavaException
from java.lang import Object
from org.python.core import PyFunction


class RequestWatcher(object):
    def block(self):
        # type: () -> bool
        raise NotImplementedError

    def compose(self, *args):
        # type: (RequestWatcher) -> RequestWatcher
        raise NotImplementedError


class Request(RequestWatcher):
    def block(self):
        # type: () -> bool
        pass

    def cancel(self):
        # type: () -> None
        raise NotImplementedError

    def compose(self, *args):
        # type: (RequestWatcher) -> RequestWatcher
        pass

    def get(self):
        # type: () -> Object
        raise NotImplementedError

    def getError(self):
        # type: () -> JavaException
        raise NotImplementedError

    def onError(self, func):
        # type: (PyFunction) -> None
        raise NotImplementedError

    def onSuccess(self, func):
        # type: (PyFunction) -> None
        raise NotImplementedError



"""Device Functions.

The following functions give you access to view and edit device
connections in the Gateway.
"""

from __future__ import print_function

__all__ = [
    "addDevice",
    "getDeviceHostname",
    "listDevices",
    "refreshBrowse",
    "removeDevice",
    "restart",
    "setDeviceEnabled",
    "setDeviceHostname",
]

from typing import Any, Dict, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import String


def addDevice(
    deviceType,  # type: String
    deviceName,  # type: String
    deviceProps,  # type: Dict[String, Any]
    description=None,  # type: Optional[String]
):
    # type: (...) -> None
    """Adds a new device connection in Ignition.

    Accepts a dictionary of parameters to configure the connection.
    Acceptable parameters differ by device type: i.e., a Modbus/TCP
    connection requires a hostname and port, but a simulator doesn't
    require any parameters. When using this function, the arguments must
    be passed in as keyword arguments.

    Args:
        deviceType: The device driver type.
        deviceName: The name that will be given to the new device
            connection.
        deviceProps: A dictionary of device connection properties and
            values. Each deviceType has different properties, but most
            require at least a hostname. Keys in the dictionary are
            case-insensitive, spaces are omitted, and the names of the
            properties that appear when manually creating a device
            connection.
        description: The description that will be given to the new
            device connection. Optional.
    """
    print(deviceType, deviceName, deviceProps, description)


def getDeviceHostname(deviceName):
    # type: (String) -> String
    """Gets the hostname of a device.

    Args:
        deviceName: The name of the device in Ignition.

    Returns:
        The hostname of the device. Null if device doesn't have a
        hostname.
    """
    print(deviceName)
    return ""


def listDevices():
    # type: () -> BasicDataset
    """Returns a dataset of information about each configured device.

    Each row represents a single device.

    Returns:
        A dataset, where each row represents a device. Contains 4
        columns Name, Enabled, State, and Driver.
    """
    return BasicDataset()


def refreshBrowse(deviceName):
    # type: (String) -> None
    """Forces Ignition to browse the controller.

    Only works for Allen-Bradley controllers.

    Args:
        deviceName: The name of the device in Ignition.
    """
    print(deviceName)


def removeDevice(deviceName):
    # type: (String) -> None
    """Removes a given device from Ignition.

    Args:
        deviceName: The name of the device in Ignition.
    """
    print(deviceName)


def restart(deviceName):
    # type: (String) -> None
    """Restarts the named device connection.

    Args:
        deviceName: The name of the device connection to restart. The
            function will throw an error if the specified deviceName
            does not exist on the host gateway.
    """
    print(deviceName)


def setDeviceEnabled(deviceName, enabled):
    # type: (String, bool) -> None
    """Enables/disables a device in Ignition.

    Args:
        deviceName: The name of the device in Ignition.
        enabled: Specifies whether the device connection will be set to
            enabled or disabled state.
    """
    print(deviceName, enabled)


def setDeviceHostname(deviceName, hostname):
    # type: (String, String) -> None
    """Changes the hostname of a device.

    Used for all Ethernet based drivers.

    Args:
        deviceName: The name of the device in Ignition.
        hostname: The new IP address or hostname.
    """
    print(deviceName, hostname)



__all__ = ["ApplicationScope", "Version"]

import re

from typing import Any, Optional, Tuple, Union

from java.io import InputStream
from java.lang import IllegalArgumentException, Object, String


class ApplicationScope(Object):
    ALL = 7
    CLIENT = 4
    DESIGNER = 2
    GATEWAY = 1
    NONE = 0

    @staticmethod
    def getGlobalScope():
        # type: () -> int
        """Returns the scope of this jvm, wherever it may be.

        Returns:
            The scope of this JVM, wherever it may be.

        """
        return -1

    @staticmethod
    def getScopePrefix():
        # type: () -> String
        """Returns the current jvm's scope prefix, for use in thread
        names.

        Returns:
            The current JVM's scope prefix, for use in thread names.
        """
        return "designer"

    @staticmethod
    def init(globalScope):
        # type: (int) -> None
        pass

    @staticmethod
    def isClient(scope):
        # type: (int) -> bool
        pass

    @staticmethod
    def isDesigner(scope):
        # type: (int) -> bool
        pass

    @staticmethod
    def isGateway(scope):
        # type: (int) -> bool
        pass

    @staticmethod
    def parseScope(s):
        # type: (String) -> int
        """Returns a bitmask representing application scope for strings
        like:

            "C"= client
            "DC"= designer or client
            "G"= gateway
            "N"=none
            "A"=all

        Args:
            s: The scope.

        Returns:
            The scope's int value.
        """
        pass

    @staticmethod
    def toCode(scope):
        # type: (int) -> String
        """Turns a scope int into the various scope codes:

            "C"= client
            "DC"= designer or client
            "G"= gateway
            "N"=none
            "A"=all

        Args:
            scope: The scope's int value.

        Returns:
            The scope code.
        """
        pass


class Version(Object):
    build = 0  # type: int
    dev = True  # type: bool
    major = 0  # type: int
    minor = 0  # type: int
    rev = 0  # type: int
    beta = 0  # type: int
    rc = 0  # type: int
    snapshot = False  # type: bool

    def __init__(self, major=0, minor=0, rev=0, build=0, beta=0, rc=0):
        # type: (int, int, int, int, int, int) -> None
        self.major = major
        self.minor = minor
        self.rev = rev
        self.build = build
        self.beta = beta
        self.rc = rc

    def __eq__(self, other):
        # type: (Any) -> bool
        return self._compare(other, True) == 0

    def __str__(self):
        # type: () -> str
        return self.toString()

    def _compare(self, that, strict=False):
        # type: (Version, bool) -> int
        _this = self.toTuple(strict)
        _that = that.toTuple(strict)
        return (_this > _that) - (_this < _that)

    def compareTo(self, that):
        # type: (Version) -> Optional[int]
        return self._compare(that, True) if isinstance(that, type(self)) else None

    def exists(self):
        # type: () -> bool
        pass

    @staticmethod
    def fromXML(inputStream):
        # type: (InputStream) -> Version
        pass

    def getBasicString(self):
        # type: () -> str
        return (
            "{}.{}.{}-rc{}".format(self.major, self.minor, self.rev, self.rc)
            if self.rc > 0
            else "{}.{}.{}".format(self.major, self.minor, self.rev)
        )

    def getBeta(self):
        # type: () -> int
        return self.beta

    def getBuildNumber(self):
        # type: () -> int
        return self.build

    def getMajor(self):
        # type: () -> int
        return self.major

    def getMinor(self):
        # type: () -> int
        return self.minor

    def getRc(self):
        # type: () -> int
        return self.rc

    def getRevision(self):
        # type: () -> int
        return self.rev

    def getXML(self):
        # type: () -> str
        pass

    def isDev(self):
        # type: () -> bool
        return self.dev

    def isFutureVersion(self, arg):
        # type: (Union[Version, String]) -> bool
        cls = type(self)
        if isinstance(arg, basestring):
            that = self.parse(arg)
        elif isinstance(arg, cls):
            that = arg
        else:
            raise TypeError("isFutureVersion(): 1st arg can't be coerced to String.")

        return self._compare(that) == -1

    def isSnapshot(self):
        # type: () -> bool
        return self.snapshot

    @staticmethod
    def parse(s):
        # type: (String) -> Version
        sem_ver = [int(i) for i in re.findall(r"-?\d+", s)]
        if len(sem_ver) < 3:
            raise IllegalArgumentException('Invalid version: "{}"'.format(s))
        build_number = sem_ver[3] if len(sem_ver) == 4 else 0
        return Version(sem_ver[0], sem_ver[1], sem_ver[2], build_number)

    def toParseableString(self):
        # type: () -> str
        return "{}.{}.{}.{}".format(self.major, self.minor, self.rev, self.build)

    def toString(self):
        # type: () -> str
        if self.rc > 0:
            return "{}.{}.{}-rc{} (b{})".format(
                self.major, self.minor, self.rev, self.rc, self.build
            )
        if self.isSnapshot():
            return "{}.{}.{}-SNAPSHOT (b{})".format(
                self.major, self.minor, self.rev, self.build
            )
        if self.build is not None:
            return "{}.{}.{} (b{})".format(self.major, self.minor, self.rev, self.build)

        return "{}.{}.{}".format(self.major, self.minor, self.rev)

    def toTuple(self, strict=False):
        # type: (bool) -> Tuple[int, ...]
        return (
            (self.major, self.minor, self.rev, self.build, self.beta, self.rc)
            if strict
            else (self.major, self.minor, self.rev)
        )



__all__ = [
    "AttributeView",
    "BasicFileAttributes",
    "FileAttribute",
    "FileAttributeView",
    "FileStoreAttributeView",
    "FileTime",
    "PosixFilePermission",
    "UserPrincipal",
]

from typing import List, TypeVar

from java.lang import Enum, Object, String
from java.security import Principal
from java.time import Instant
from java.util.concurrent import TimeUnit

T = TypeVar("T")


class AttributeView(object):
    def name(self):
        # type: () -> String
        raise NotImplementedError


class BasicFileAttributes(object):
    def creationTime(self):
        # type: () -> FileTime
        raise NotImplementedError

    def fileKey(self):
        # type: () -> Object
        raise NotImplementedError

    def isDirectory(self):
        # type: () -> bool
        raise NotImplementedError

    def isOther(self):
        # type: () -> bool
        raise NotImplementedError

    def isRegularFile(self):
        # type: () -> bool
        raise NotImplementedError

    def isSymbolicLink(self):
        # type: () -> bool
        raise NotImplementedError

    def lastAccessTime(self):
        # type: () -> FileTime
        raise NotImplementedError

    def lastModifiedTime(self):
        # type: () -> FileTime
        raise NotImplementedError

    def size(self):
        # type: () -> long
        raise NotImplementedError


class FileAttribute(object):
    def name(self):
        # type: () -> String
        pass

    def value(self):
        # type: () -> T
        pass


class FileAttributeView(AttributeView):
    def name(self):
        # type: () -> String
        pass


class FileTime(Object):
    def compareTo(self, other):
        # type: (FileTime) -> int
        pass

    @staticmethod
    def fromMillis(value):
        # type: (long) -> FileTime
        pass

    def to(self, unit):
        # type: (TimeUnit) -> long
        pass

    def toInstant(self):
        # type: () -> Instant
        pass

    def toMillis(self):
        # type: () -> long
        pass


class PosixFilePermission(Enum):
    GROUP_EXECUTE = None  # type: PosixFilePermission
    GROUP_READ = None  # type: PosixFilePermission
    GROUP_WRITE = None  # type: PosixFilePermission
    OTHERS_EXECUTE = None  # type: PosixFilePermission
    OTHERS_READ = None  # type: PosixFilePermission
    OTHERS_WRITE = None  # type: PosixFilePermission
    OWNER_EXECUTE = None  # type: PosixFilePermission
    OWNER_READ = None  # type: PosixFilePermission
    OWNER_WRITE = None  # type: PosixFilePermission

    @staticmethod
    def values():
        # type: () -> List[PosixFilePermission]
        pass


class UserPrincipal(Principal):
    def equals(self, another):
        # type: (Object) -> bool
        pass

    def getName(self):
        # type: () -> String
        pass

    def hashCode(self):
        # type: () -> int
        pass

    def toString(self):
        # type: () -> String
        pass


class FileStoreAttributeView(AttributeView):
    def name(self):
        # type: () -> String
        pass



from java.lang import Object, String
from javax.security.auth import Subject


class Principal(object):
    def equals(self, another):
        # type: (Object) -> bool
        raise NotImplementedError

    def getName(self):
        # type: () -> String
        raise NotImplementedError

    def hashCode(self):
        # type: () -> int
        raise NotImplementedError

    def implies(self, subject):
        # type: (Subject) -> bool
        pass

    def toString(self):
        # type: () -> String
        raise NotImplementedError



class ClientContext(object):
    def addPropertyChangeListener(self, *args):
        raise NotImplementedError

    def desearilize(self, data, log):
        raise NotImplementedError

    def getAuthProfileName(self):
        raise NotImplementedError

    def getDefaultDataSourceName(self):
        raise NotImplementedError

    def getDefaultTagProviderName(self):
        raise NotImplementedError

    def getExecutionManager(self):
        raise NotImplementedError

    def getLaunchContext(self):
        raise NotImplementedError

    def getLocalizationManager(self):
        raise NotImplementedError

    def getModules(self):
        raise NotImplementedError

    def getNamedQueryManager(self):
        raise NotImplementedError

    def getProgressManager(self):
        raise NotImplementedError

    def getProject(self):
        raise NotImplementedError

    def getProjectName(self):
        raise NotImplementedError

    def getRootPaneContainer(self):
        raise NotImplementedError

    def getTagManager(self):
        raise NotImplementedError

    def getTagPollRate(self):
        raise NotImplementedError

    def getUIEventBus(self):
        raise NotImplementedError

    def removePropertyChangeListened(self, *args):
        raise NotImplementedError



"""Perspective Functions.

The following functions offer various ways to interact with a
Perspective Session from a Python script.
"""

from __future__ import print_function

__all__ = [
    "alterDock",
    "alterLogging",
    "authenticationChallenge",
    "closeDock",
    "closePage",
    "closePopup",
    "closeSession",
    "download",
    "getProjectInfo",
    "getSessionInfo",
    "isAuthorized",
    "login",
    "logout",
    "navigate",
    "navigateBack",
    "navigateForward",
    "openDock",
    "openPopup",
    "print",
    "refresh",
    "sendMessage",
    "setTheme",
    "toggleDock",
    "togglePopup",
    "vibrateDevice",
]

import __builtin__ as builtins

from typing import Any, Dict, List, Optional, Union

from com.inductiveautomation.ignition.common.gson import JsonObject
from com.inductiveautomation.ignition.common.script.adapters import PyJsonObjectAdapter
from java.lang import String


def alterDock(
    dockId,  # type: String
    config=None,  # type: Optional[Dict[String, Any]]
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
):
    # type: (...) -> None
    """Changes configuration of a specified dock on a Perspective Page.

    Args:
        dockId: The ID of the dock to be altered. If no such dock exists
            on the current page with the given ID, a warning will be
            logged to the console.
        config: The new configuration for the dock. All properties are
            optional, and any properties not specified will remain
            unchanged. Optional.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(dockId, config, sessionId, pageId)


def alterLogging(
    remoteLoggingEnabled=False,  # type: bool
    level="info",  # type: String
    remoteLoggingLevel="warn",  # type: String
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
):
    # type: (...) -> None
    """Changes Perspective Session logging attributes and levels.

    All parameters are optional, with the caveat that at least one of
    them needs to be used.

    Args:
        remoteLoggingEnabled: Will enable remote logging if True. Remote
            logging will send log events from the Session to the Gateway
            under the `perspective.client` logger if they meet the
            remoteLevel logging level requirement. Optional.
        level: The desired Session logging level. Possible values are:
            all, trace, debug, info, warn, error, fatal, off. The
            default is info. Optional.
        remoteLoggingLevel: The desired remote logging level. Possible
            values are: all, trace, debug, info, warn, error, fatal,
            off. The default is warn. Optional.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(remoteLoggingEnabled, level, remoteLoggingLevel, sessionId, pageId)


def authenticationChallenge(
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    idp="",  # type: String
    forceAuth=False,  # type: bool
    timeout=2,  # type: int
    payload=None,  # type: Optional[Dict[String, Any]]
    framing="self",  # type: String
):
    # type: (...) -> None
    """Triggers an authentication challenge action.

    Args:
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the page to target. If omitted, the
            current page will be used. Optional.
        idp: The name of the IdP to use for this authentication
            challenge. If omitted, the Project default IdP will be used.
            Optional.
        forceAuth: True if Ignition should ask the IdP to
            re-authenticate the user, even if the user is already signed
            into the IdP. False if Ignition should not ask the IdP to
            re-authenticate the user. If the IdP supports this option,
            the IdP will ask the user to re-enter their credentials,
            even if the user is already signed into the IdP. If omitted,
            the default value for this argument will fall back to the
            value in the Project Properties. Optional.
        timeout: The number of minutes the system will wait in between
            the authentication request and the authentication response
            before timing out the request. If set to any number <= zero,
            the request is rejected. If omitted, the default of two
            minutes will be used as the timeout. Optional.
        payload: An opaque payload object that may contain any
            information. This object will be passed to the
            onAuthenticationChallengeCompleted session event script. The
            payload should be a JSON-encodable data structure.
            Optional.
        framing: A string representing the type of framing that should
            be used. A value of "self" indicates that the same window
            should be used. A value of "new" indicates that a new tab
            should be used. A value of "embedded" indicates that an
            embedded iframe should be used. If omitted, the default
            value of "self" (same window) is used. Optional.
    """
    builtins.print(sessionId, pageId, idp, forceAuth, timeout, payload, framing)


def closeDock(id, sessionId="current_session", pageId="current_page"):
    # type: (String, String, String) -> None
    """Closes a docked view.

    Args:
        id: The unique, preconfigured dock ID for the docked View. Is
            specified when a View is assigned as docked for a particular
            Page (in Page Configuration).
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(id, sessionId, pageId)


def closePage(
    message=None,  # type: Optional[String]
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
):
    # type: (...) -> None
    """Closes the page with the given page id or the current page if no
    page id is provided.

    If a message is provided, it is displayed on the page when the page
    closes. Otherwise, the default message (set in the Project
    Properties) is displayed.

    Args:
        message: The message to display when the page closes. If
            omitted, the default message (set in the Project Properties)
            is shown. Optional.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the page to be closed. If omitted, the
            current pageId is used. Optional.
    """
    builtins.print(message, sessionId, pageId)


def closePopup(id, sessionId="current_session", pageId="current_page"):
    # type: (String, String, String) -> None
    """Closes a popup View.

    Args:
        id: The unique identifier for the popup, given to the popup when
            first opened. If given an empty string, then the most
            recently focused popup will be closed.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(id, sessionId, pageId)


def closeSession(message=None, sessionId="current_session"):
    # type: (Optional[String], String) -> None
    """Closes the Perspective Session with the given Session ID or the
    current Session if no ID is provided.

    If a message is provided, it is displayed on the page when the
    Session closes. Otherwise, the default message (set in the Project
    Properties) is displayed.

    Args:
        message: The message to display when the Session closes. If
            omitted, the default message (set in the Project Properties)
            is shown. Optional.
        sessionId: Identifier of the Session to be closed. If omitted,
            the current sessionId is used. Optional.
    """
    builtins.print(message, sessionId)


def download(
    filename,  # type: String
    data,  # type: Any
    contentType=None,  # type: Optional[String]
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
):
    # type: (...) -> None
    """Downloads data from the gateway to a device running a Session.

    Args:
        filename: Suggested name for the downloaded file.
        data: The data to be downloaded. May be a String, a byte[], or
            an InputStream. Strings will be written with in "utf-8"
            encoding.
        contentType: Value for the "Content-Type" header. Example:
            "text/plain; charset=utf-8". Optional.
        sessionId: Identifier of the Session to target. If omitted the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(filename, data, contentType, sessionId, pageId)


def getProjectInfo():
    # type: () -> Dict[String, Any]
    """Returns a dictionary of meta data from a Perspective Project.

    Returns:
        A dictionary of project meta data.
    """
    return {
        "name": "Project",
        "title": "Project",
        "description": "Project's description",
        "lastModified": "2021-11-22T20:46:00.000Z",
        "lastModifiedBy": "thecesrom",
        "views": [{"path": "Page/Home"}],
        "pageConfigs": [{"url": "", "primaryView": ""}],
    }


def getSessionInfo(
    usernameFilter=None,  # type: Optional[String]
    projectFilter=None,  # type: Optional[String]
):
    # type: (...) -> List[PyJsonObjectAdapter]
    """Returns information about one or more Perspective Sessions.

    The information returned by this function is a combination of
    information available on the perspective Sessions status page on the
    Gateway, and some Session props (id and userAgent).

    Args:
        usernameFilter: A filter based on logged-in user. Optional.
        projectFilter: A filter based on the project name. Optional.

    Returns:
        A list of objects (PyJsonObjectAdapter).
    """
    builtins.print(usernameFilter, projectFilter)
    return [PyJsonObjectAdapter(JsonObject())]


def isAuthorized(isAllOf, securityLevels, sessionId="current_session"):
    # type: (bool, List[String], String) -> bool
    """Checks if the user in the current Session is authorized against a
    target collection of security levels.

    Args:
        isAllOf: True if the current user must have all of the given
            security levels to be authorized. False if the current user
            must have at least one of the given security levels to be
            authorized.
        securityLevels: An array of string paths to a security level
            node in the form of "Path/To/Node". Each level in the tree
            is delimited by a forward slash character. The public node
            is never a part of the path.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. Optional.

    Returns:
        True if the user in the current Session is authorized, False
        otherwise.
    """
    builtins.print(isAllOf, securityLevels, sessionId)
    return True


def login(sessionId="current_session", pageId="current_page", forceAuth=False):
    # type: (String, String, bool) -> None
    """Triggers a login event that will allow the user to login with the
    project's configured Identity Provider (IdP).

    For this function to work, an Identity Provider must be set in
    Perspective project properties. This is particularly useful when you
    want it to be possible to start a Session without authentication and
    sign in to access certain restricted features.

    Args:
        sessionId: Identifier of the Session to target. If omitted the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        forceAuth: Determines if Ignition should ask the Identity
            Provider to re-authenticate the user, even if the user is
            already signed into the Identity Provider. If set to True,
            then the Identity Provider will ask the user to re-enter
            their credentials. If set to False, then the Gateway will
            request that the Identity Provider use the last provided
            credentials for the Session, potentially allowing
            re-authentication without requiring the user to re-type
            their credentials. Note that support for this argument is
            determined by the Identity Provider; the IdP may choose to
            ignore this request. If this parameter is omitted, then the
            function will use the re-authentication setting defined
            under Project Properties. Optional.
    """
    builtins.print(sessionId, pageId, forceAuth)


def logout(
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    message="default message",  # type: String
):
    # type: (...) -> None
    """Triggers a logout event, which will log the user out.

    For this function to work, an Identity Provider must be set in the
    Perspective project properties.

    Args:
        sessionId: Identifier of the Session to target. If omitted the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        message: The message to display when the user logs out of their
            session. This message is only shown when the project
            requires authentication. If omitted, the default message
            (set in Project Properties) is shown. Optional.
    """
    builtins.print(sessionId, pageId, message)


def navigate(
    page,  # type: String
    url=None,  # type: Optional[String]
    view=None,  # type: Optional[String]
    params=None,  # type: Optional[Dict[String, String]]
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    newTab=False,  # type: bool
):
    # type: (...) -> None
    """Navigate the Session to a specified view or mounted page.

    The function can be used in three different ways, depending on which
    parameter is specified:
        page: navigates to a perspective-page
        url: navigates to a Web address, so the function can be used to
            navigate the user to a Web portal, search engine, or other
            Website. This parameter is specified via keyword argument
        view: navigates to a view. Note that using this parameter does
            not modify the Web browser's address bar, so the browser's
            history will not contain a listing for the new view. This
            parameter is specified via keyword argument

    Args:
        page: The URL of a Perspective page to navigate to.
        url: The URL of a Web address to navigate to. If the page or
            view parameters are specified, then this parameter is
            ignored. Optional.
        view: If specified, will navigate to a specific view. Navigating
            to a view via this parameter does not change the address in
            the Web browser. Thus the Web browser's back button will not
            be able to return the user to the previous view. If the page
            parameter is specified, then this parameter is ignored.
            Optional.
        params: Used only in conjunction with the view parameter,
            Dictionary of values to pass to any parameters on the view.
            Optional.
        sessionId: Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different Session, then the pageId parameter
            must be included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        newTab: If True, opens the contents in a new tab.
            Optional.
    """
    builtins.print(page, url, view, params, sessionId, pageId, newTab)


def navigateBack(sessionId="current_session", pageId="current_page"):
    # type: (String, String) -> None
    """Navigate the Session to a specified view or mounted page.

    This is similar to a browser's "back" function.

    Args:
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. Optional.
        pageId: Identifier of the page to target. If omitted, the
            current page will be used automatically. Optional.
    """
    builtins.print(sessionId, pageId)


def navigateForward(sessionId="current_session", pageId="current_page"):
    # type: (String, String) -> None
    """Navigate the Session to a specified view or mounted page.

    This is similar to a browser's "forward" function.

    Args:
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the page to target. If omitted, the
            current page will be used automatically. Optional.
    """
    builtins.print(sessionId, pageId)


def openDock(
    id,  # type: String
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    params=None,  # type: Optional[Dict[String, String]]
):
    # type: (...) -> None
    """Opens a docked View.

    Requires the preconfigured dock ID for the view.

    Args:
        id: The unique, preconfigured 'Dock ID' for the docked View. Is
            specified when a View is assigned as docked for a particular
            Page (in Page Configuration).
        sessionId: Identifier of the Session to target. If omitted the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        params: Parameters that can be passed into the docked view. Must
            match the docked views View Parameters. Optional.
    """
    builtins.print(id, sessionId, pageId, params)


def openPopup(
    id,  # type: String
    view,  # type: String
    params=None,  # type: Optional[Dict[String, Any]]
    title="",  # type: String
    position=None,  # type: Optional[Dict[String, Union[int, String]]]
    showCloseIcon=True,  # type: bool
    draggable=True,  # type: bool
    resizable=False,  # type: bool
    modal=False,  # type: bool
    overlayDismiss=False,  # type: bool
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    viewPortBound=False,  # type: bool
):
    # type: (...) -> None
    """Open a popup view over the given page.

    Args:
        id: A unique popup string. Will be used to close the popup from
            other popup or script actions.
        view: The path to the View to use in the popup.
        params: Dictionary of key-value pairs to us as input parameters
            to the View. Optional.
        title: Text to display in the title bar. If no value or an empty
            string are given, the title bar will not be displayed.
            Defaults to an empty string. Optional.
        position: Dictionary of key-value pairs to use for position.
            Possible position keys are: left, top, right, bottom, width,
            height. Defaults to the center of the window. Optional.
        showCloseIcon: Will show the close icon if True. Defaults to
            True. Optional.
        draggable: Will allow the popup to be dragged if True. Defaults
            to True. Optional.
        resizable: Will allow the popup to be resized if True. Defaults
            to False. Optional.
        modal: Will make the popup modal if True. A modal popup is the
            only view the user can interact with. Defaults to False.
            Optional.
        overlayDismiss: Will allow the user to dismiss and close a modal
            popup by clicking outside of it if True. Defaults to False.
            Optional.
        sessionId: Identifier of the Session to target. If omitted the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        viewPortBound: If True, popups will be "shifted" to open within
            the bounds of the viewport. If the popup would be larger
            than the viewport, then it will be resized to fit within the
            bounds. Default is False. Optional.
    """
    builtins.print(
        id,
        view,
        params,
        title,
        position,
        showCloseIcon,
        draggable,
        resizable,
        modal,
        overlayDismiss,
        sessionId,
        pageId,
        viewPortBound,
    )


def print(
    message,  # type: String
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    destination="client",  # type: String
):
    # type: (...) -> None
    """Sends print statements to the scripting console when in the
    Designer.

    When in a Session, sends print statements to the output console.
    This function makes scripting diagnostics easier.

    Args:
        message: The print statement that will be displayed on the
            console.
        sessionId: Identifier of the Session to target. If omitted the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        destination: Where the message should be printed. If specified,
            must be "client", "gateway", or "all". Default is "client".
            Optional.
    """
    builtins.print(message, sessionId, pageId, destination)


def refresh(sessionId="current_session", pageId="current_page"):
    # type: (Optional[String], Optional[String]) -> None
    """Triggers a refresh of the page.

    Note:
        This method should not be confused with the refreshBinding
        component method, which automatically fires a binding on a
        Perspective component property.

    Args:
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(sessionId, pageId)


def sendMessage(
    messageType,  # type: String
    payload,  # type: Dict[String, String]
    scope="page",  # type: String
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
):
    # type: (...) -> None
    """Send a message to a message handler within the same Session.

    Args:
        messageType: The message type that will be invoked. Message
            handlers configured within the project are listening for
            messages of a specific messageType.
        payload: A python dictionary representing any parameters that
            will be passed to the message handler.
        scope: The scope that the message should be delivered to. Valid
            values are "Session", "page", or "view". If omitted, "page"
            will be used. Optional.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used. Optional.
    """
    builtins.print(messageType, payload, scope, sessionId, pageId)


def setTheme(name, sessionId="current_session", pageId="current_page"):
    # type: (String, Optional[String], Optional[String]) -> None
    """Changes the theme in a page to the specified theme.

    Note that this function only changes the theme for a single page,
    not the entire session. To change the theme for a session, write
    directly to the session.theme property instead.

    Args:
        name: The theme name to switch to. Possible values are "dark" or
            "light".
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(name, sessionId, pageId)


def toggleDock(
    id,  # type: String
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    params=None,  # type: Optional[Dict[String, String]]
):
    # type: (...) -> None
    """Toggles a docked View.

    Args:
        id: The unique, preconfigured 'Dock ID' for the docked View. Is
            specified when a View is assigned as docked for a particular
            Page (in Page Configuration).
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        params: Parameters that can be passed into the docked view. Must
            match the docked views View Parameters. Optional.
    """
    builtins.print(id, sessionId, pageId, params)


def togglePopup(
    id,  # type: String
    view,  # type: String
    params,  # type: Optional[Dict[String, Any]]
    title="",  # type: String
    position=None,  # type: Optional[Dict[String, Union[int, String]]]
    showCloseIcon=True,  # type: bool
    draggable=True,  # type: bool
    resizable=False,  # type: bool
    modal=False,  # type: bool
    overlayDismiss=False,  # type: bool
    sessionId="current_session",  # type: String
    pageId="current_page",  # type: String
    viewPortBound=False,  # type: bool
):
    # type: (...) -> None
    """Toggles a popup view.

    Will open up the popup if it has not been opened yet. Otherwise, it
    will close the currently opened popup.

    Args:
        id: A unique popup string. Will be used to close the popup from
            other popup or script actions.
        view: The path to the View to use in the popup.
        params: Dictionary of key-value pairs to us as input parameters
            to the View. Optional.
        title: Text to display in the title bar. If no value or an empty
            string are given, the title bar will not be displayed.
            Defaults to an empty string. Optional.
        position: Dictionary of key-value pairs to use for position.
            Possible position keys are: left, top, right, bottom, width,
            height. Defaults to the center of the window. Optional.
        showCloseIcon: Will show the close icon if True. Defaults to
            True. Optional.
        draggable: Will allow the popup to be dragged if True. Defaults
            to True. Optional.
        resizable: Will allow the popup to be resized if True. Defaults
            to False. Optional.
        modal: Will make the popup modal if True. A modal popup is the
            only view the user can interact with. Defaults to False.
            Optional.
        overlayDismiss: Will allow the user to dismiss and close a modal
            popup by clicking outside of it if True. Defaults to False.
            Optional.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. When targeting a
            different Session, then the pageId parameter must be
            included in the call. Optional.
        pageId: Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        viewPortBound: If True, popups will be "shifted" to open within
            the bounds of the viewport. If the popup would be larger
            than the viewport, then it will be resized to fit within the
            bounds. Default is False. Optional.
    """
    builtins.print(
        id,
        view,
        params,
        title,
        position,
        showCloseIcon,
        draggable,
        resizable,
        modal,
        overlayDismiss,
        sessionId,
        pageId,
        viewPortBound,
    )


def vibrateDevice(duration, sessionId="current_session"):
    # type: (int, Optional[String]) -> None
    """When called from the Perspective App, will cause the
    device to vibrate for the specified number of milliseconds.

    Note:
        iOS vibration duration is fixed. This function will cause an iOS
        device to vibrate for its default duration, 0.4 seconds
        (400 milliseconds).

    Args:
        duration: The duration in milliseconds to vibrate the device.
        sessionId: Identifier of the Session to target. If omitted, the
            current Session will be used automatically. Optional.
    """
    builtins.print(duration, sessionId)



from __future__ import print_function

__all__ = ["AlarmEvent", "EventData", "PyAlarmEvent", "PyAlarmEventImpl"]

from typing import Any, Iterator

from com.inductiveautomation.ignition.common.config import BasicPropertySet
from org.python.core import PyObject


class AlarmEvent(object):
    def __iter__(self):
        # type: () -> Iterator[Any]
        raise NotImplementedError

    def acknowledge(self, ackData):
        # type: (EventData) -> None
        raise NotImplementedError

    def active(self, activeData):
        raise NotImplementedError

    def clear(self, clearData):
        raise NotImplementedError

    def getAckData(self):
        raise NotImplementedError

    def getActiveData(self):
        raise NotImplementedError

    def getClearedData(self):
        raise NotImplementedError

    def getDisplayPath(self):
        raise NotImplementedError

    def getId(self):
        raise NotImplementedError

    def getLabel(self):
        raise NotImplementedError

    def getLastEventState(self):
        raise NotImplementedError

    def getName(self):
        raise NotImplementedError

    def getNotes(self):
        raise NotImplementedError

    def getPriority(self):
        raise NotImplementedError

    def getSource(self):
        raise NotImplementedError

    def getState(self):
        raise NotImplementedError

    def isAcked(self):
        raise NotImplementedError

    def isCleared(self):
        raise NotImplementedError

    def isShelved(self):
        raise NotImplementedError

    def iterator(self):
        pass


class EventData(BasicPropertySet):
    def __init__(self, *args):
        # type: (*Any) -> None
        super(EventData, self).__init__(*args)

    def getTimeStamp(self):
        # type: () -> long
        pass


class PyAlarmEvent(AlarmEvent):
    def __iter__(self):
        # type: () -> Iterator[Any]
        pass

    def acknowledge(self, ackData):
        pass

    def active(self, activeData):
        pass

    def clear(self, clearData):
        pass

    def contains(self, property):
        raise NotImplementedError

    def get(self, property):
        raise NotImplementedError

    def getAckData(self):
        pass

    def getActiveData(self):
        pass

    def getClearedData(self):
        pass

    def getDisplayPath(self):
        pass

    def getId(self):
        pass

    def getLabel(self):
        pass

    def getLastEventState(self):
        pass

    def getName(self):
        pass

    def getNotes(self):
        pass

    def getOrDefault(self, property):
        raise NotImplementedError

    def getOrElse(self, property, defaultValue):
        raise NotImplementedError

    def getPriority(self):
        pass

    def getSource(self):
        pass

    def getState(self):
        pass

    def isAcked(self):
        pass

    def isCleared(self):
        pass

    def isShelved(self):
        pass

    def set(self, property, value):
        raise NotImplementedError

    def setGlobal(self, property, value):
        raise NotImplementedError

    def sourceEvent(self):
        raise NotImplementedError


class PyAlarmEventImpl(PyAlarmEvent, PyObject):
    def __init__(self, event):
        print(event)
        super(PyAlarmEventImpl, self).__init__()

    def contains(self, property):
        pass

    def get(self, property):
        pass

    def getOrDefault(self, property):
        pass

    def getOrElse(self, property, defaultValue):
        pass

    def set(self, property, value):
        pass

    def setGlobal(self, property, value):
        pass

    def sourceEvent(self):
        pass



__all__ = ["Clock", "Duration", "Instant", "ZoneId", "ZoneOffset"]

from typing import Dict, Optional, Set

from java.lang import CharSequence, Object, String


class Clock(Object):
    @staticmethod
    def offset(baseClock, offsetDuration):
        # type: (Instant, ZoneId) -> Clock
        pass

    @staticmethod
    def systemDefaultZone():
        # type: () -> Clock
        pass

    @staticmethod
    def systemUTC():
        # type: () -> Clock
        pass

    @staticmethod
    def tick(baseClock, tickDuration):
        # type: (Clock, Duration) -> Clock
        pass

    @staticmethod
    def tickMillis(zone):
        # type: (ZoneId) -> Clock
        pass

    @staticmethod
    def tickMinutes(zone):
        # type: (ZoneId) -> Clock
        pass

    @staticmethod
    def tickSeconds(zone):
        # type: (ZoneId) -> Clock
        pass


class Duration(Object):
    ZERO = None  # type: Duration

    def toDays(self):
        # type: () -> long
        pass

    def toDaysPart(self):
        # type: () -> long
        pass

    def toHours(self):
        # type: () -> long
        pass

    def toHoursPart(self):
        # type: () -> int
        pass

    def toMillis(self):
        # type: () -> long
        pass

    def toMillisPart(self):
        # type: () -> int
        pass

    def toMinutes(self):
        # type: () -> long
        pass

    def toMinutesPart(self):
        # type: () -> int
        pass

    def toNanos(self):
        # type: () -> long
        pass

    def toNanosPart(self):
        # type: () -> int
        pass

    def toSeconds(self):
        # type: () -> long
        pass

    def toSecondsPart(self):
        # type: () -> int
        pass


class Instant(Object):
    EPOCH = None  # type: Instant
    MAX = None  # type: Instant
    MIN = None  # type: Instant

    @staticmethod
    def now(clock=None):
        # type: (Optional[Clock]) -> Instant
        pass

    @staticmethod
    def ofEpochMilli(epochMilli):
        # type: (long) -> Instant
        pass

    @staticmethod
    def ofEpochSecond(epochSecond, nanoAdjustment=None):
        # type: (long, Optional[long]) -> Instant
        pass

    @staticmethod
    def parse(text):
        # type: (CharSequence) -> Instant
        pass


class ZoneId(Object):
    SHORT_IDS = None  # type: Dict[String, String]

    @staticmethod
    def getAvailableZoneIds():
        # type: () -> Set[String]
        pass

    @staticmethod
    def of(zoneId, aliasMap=None):
        # type: (String, Optional[Dict[String, String]]) -> ZoneId
        pass

    @staticmethod
    def ofOffset(prefix, offset):
        # type: (String, ZoneOffset) -> ZoneId
        pass

    @staticmethod
    def systemDefault():
        # type: () -> ZoneId
        pass


class ZoneOffset(Object):
    MAX = None  # type: ZoneOffset
    MIN = None  # type: ZoneOffset
    UTC = None  # type: ZoneOffset

    @staticmethod
    def of(offsetId):
        # type: (String) -> ZoneOffset
        pass

    @staticmethod
    def ofHours(hours):
        # type: (int) -> ZoneOffset
        pass

    @staticmethod
    def ofHoursMinutes(hours, minutes):
        # type: (int, int) -> ZoneOffset
        pass

    @staticmethod
    def ofHoursMinutesSeconds(hours, minutes, seconds):
        # type: (int, int, int) -> ZoneOffset
        pass

    @staticmethod
    def ofTotalSeconds(totalSeconds):
        # type: (int) -> ZoneOffset
        pass



"""Provides for system input and output through data streams,
serialization and the file system. Unless otherwise noted, passing a
null argument to a constructor or method in any class or interface in
this package will cause a NullPointerException to be thrown.
"""

from __future__ import print_function

__all__ = [
    "BufferedReader",
    "BufferedWriter",
    "Closeable",
    "DataOutputStream",
    "File",
    "FileDescriptor",
    "FileOutputStream",
    "FilterOutputStream",
    "Flushable",
    "IOException",
    "InputStream",
    "OutputStream",
    "PrintStream",
    "PrintWriter",
    "Reader",
    "Writer",
]

from typing import Any, Optional, Union

from java.lang import (
    Appendable,
    AutoCloseable,
    CharSequence,
    Exception,
    Object,
    String,
    Throwable,
)


class Closeable(AutoCloseable):
    def close(self):
        # type: () -> None
        raise NotImplementedError


class Flushable(object):
    def flush(self):
        # type: () -> None
        raise NotImplementedError


class File(Object):
    pathSeparator = None  # type: String
    pathSeparatorChar = None  # type: String
    separator = None  # type: String
    separatorChar = None  # type: String

    def __init__(self, *args):
        # type: (*Any) -> None
        pass


class FileDescriptor(Object):
    def sync(self):
        # type: () -> None
        pass

    def valid(self):
        # type: () -> bool
        pass


class OutputStream(Object, Closeable, Flushable):
    def close(self):
        # type: () -> None
        pass

    def flush(self):
        # type: () -> None
        pass

    @staticmethod
    def nullOutputStream():
        # type: () -> OutputStream
        pass

    def write(self, *args):
        # type: (*Any) -> None
        pass


class FileOutputStream(OutputStream):
    def __init__(self, *args):
        # type: (*Any) -> None
        pass

    def getChannel(self):
        # type: () -> Any
        pass

    def getFD(self):
        # type: () -> FileDescriptor
        pass


class FilterOutputStream(OutputStream):
    _out = None  # type: OutputStream

    def __init__(self, out):
        # type: (OutputStream) -> None
        self._out = out


class DataOutputStream(FilterOutputStream):
    out = None  # type: OutputStream

    def __init__(self, out):
        # type: (OutputStream) -> None
        self.out = out
        super(DataOutputStream, self).__init__(out)

    def size(self):
        # type: () -> int
        pass

    def writeBoolean(self, v):
        # type: (bool) -> None
        pass

    def writeByte(self, v):
        # type: (int) -> None
        pass

    def writeBytes(self, s):
        # type: (String) -> None
        pass

    def writeChar(self, v):
        # type: (int) -> None
        pass

    def writeChars(self, s):
        # type: (String) -> None
        pass

    def writeDouble(self, v):
        # type: (float) -> None
        pass

    def writeFloat(self, v):
        # type: (float) -> None
        pass

    def writeInt(self, v):
        # type: (int) -> None
        pass

    def writeLong(self, v):
        # type: (long) -> None
        pass

    def writeShort(self, v):
        # type: (int) -> None
        pass

    def writeUTF(self, s):
        # type: (String) -> None
        pass


class PrintStream(FilterOutputStream):
    _out = OutputStream()

    def __init__(self, *args):
        # type: (*Any) -> None
        print(args)
        super(PrintStream, self).__init__(self._out)

    def append(self, *args):
        # type: (*Any) -> PrintStream
        pass

    def checkError(self):
        # type: () -> bool
        pass

    def format(self, *args):
        # type: (*Any) -> PrintStream
        pass

    def print(self, arg):
        # type: (Any) -> None
        pass

    def printf(self, *args):
        # type: (*Any) -> None
        pass

    def println(self, arg):
        # type: (Any) -> None
        pass


class InputStream(Object, Closeable):
    def available(self):
        # type: () -> int
        pass

    def close(self):
        # type: () -> None
        pass

    def mark(self, readlimit):
        # type: (int) -> None
        pass

    def markSupported(self):
        # type: () -> bool
        pass

    @staticmethod
    def nullInputStream():
        # type: () -> InputStream
        pass

    def read(self, *args):
        # type: (*Any) -> int
        pass

    def readAllBytes(self):
        # type: () -> bytearray
        pass

    def readNBytes(self, *args):
        # type: (*Any) -> int
        pass

    def reset(self):
        # type: () -> None
        pass

    def skip(self, n):
        # type: (long) -> long
        pass

    def transferTo(self, out):
        # type: (OutputStream) -> long
        pass


class IOException(Exception):
    def __init__(self, message=None, cause=None):
        # type: (Optional[str], Optional[Throwable]) -> None
        super(IOException, self).__init__(message, cause)


class Reader(Object, AutoCloseable):
    def close(self):
        # type: () -> None
        raise NotImplementedError

    def mark(self, readAheadLimit):
        # type: (int) -> None
        pass

    def markSupported(self):
        # type: () -> bool
        pass

    @staticmethod
    def nullReader():
        # type: () -> Reader
        pass

    def read(self, *args):
        # type: (*Any) -> int
        pass

    def ready(self):
        # type: () -> bool
        pass

    def reset(self):
        # type: () -> None
        pass

    def skip(self, n):
        # type: (long) -> long
        pass

    def transferTo(self, out):
        # type: (Writer) -> long
        pass


class BufferedReader(Reader):
    def __init__(self, in_, sz=None):
        # type: (Reader, Optional[int]) -> None
        print(in_, sz)

    def close(self):
        # type: () -> None
        pass


class Writer(Object, Appendable, Closeable, Flushable):
    def append(self, c_csq, start=0, end=-1):
        # type: (Union[CharSequence, str], int, int) -> Writer
        pass

    def close(self):
        # type: () -> None
        pass

    def flush(self):
        # type: () -> None
        pass

    @staticmethod
    def nullWriter():
        # type: () -> Writer
        pass

    def write(self, *args):
        # type: (*Any) -> None
        pass


class BufferedWriter(Writer):
    def __init__(self, out, sz=None):
        # type: (Writer, Optional[int]) -> None
        print(out, sz)


class PrintWriter(Writer):
    def __init__(self, *args):
        # type: (*Any) -> None
        pass

    def append(self, c_csq, start=0, end=-1):
        # type: (Union[CharSequence, str], int, int) -> PrintWriter
        pass

    def checkError(self):
        # type: () -> bool
        pass

    def format(self, *args):
        # type: (*Any) -> PrintWriter
        pass

    def print(self, arg):
        # type: (Any) -> None
        pass

    def printf(self, *args):
        # type: (*Any) -> PrintWriter
        pass

    def println(self, arg):
        # type: (Any) -> None
        pass

    def write(self, *args):
        # type: (*Any) -> None
        pass



"""Contains the collections framework, legacy collection classes, event
model, date and time facilities, internationalization, and miscellaneous
utility classes (a string tokenizer, a random-number generator, and a
bit array).
"""

from __future__ import print_function

__all__ = [
    "Calendar",
    "Comparator",
    "Currency",
    "Date",
    "EventObject",
    "Iterator",
    "Locale",
    "Spliterator",
    "TimeZone",
    "UUID",
]

from typing import Any, Dict, List, Optional, Set, TypeVar, Union

from dev.thecesrom.utils.decorators import classproperty
from java.lang import Object, String
from java.time import Instant, ZoneId
from java.util.function import (
    Consumer,
    Function,
    ToDoubleFunction,
    ToIntFunction,
    ToLongFunction,
)

E = TypeVar("E")
T = TypeVar("T")


class Comparator(object):
    def compare(self, o1, o2):
        # type: (T, T) -> int
        raise NotImplementedError

    @staticmethod
    def comparing(keyExtractor, keyComparator):
        # type: (Function, Comparator) -> Comparator
        pass

    @staticmethod
    def comparingDouble(keyExtractor):
        # type: (ToDoubleFunction) -> Comparator
        pass

    @staticmethod
    def comparingInt(keyExtractor):
        # type: (ToIntFunction) -> Comparator
        pass

    @staticmethod
    def comparingLong(keyExtractor):
        # type: (ToLongFunction) -> Comparator
        pass

    def equals(self, obj):
        # type: (Object) -> bool
        raise NotImplementedError

    @staticmethod
    def naturalOrder():
        # type: () -> Comparator
        pass

    @staticmethod
    def nullsFirst(comparator):
        # type: (Comparator) -> Comparator
        pass

    @staticmethod
    def nullsLast(comparator):
        # type: (Comparator) -> Comparator
        pass

    def reversed(self):
        # type: () -> Comparator
        pass

    @staticmethod
    def reverseOrder():
        # type: () -> Comparator
        pass

    def thenComparing(self, *args):
        # type: (*Any) -> Comparator
        pass

    def thenComparingDouble(self, keyExtractor):
        # type: (ToDoubleFunction) -> Comparator
        pass

    def thenComparingInt(self, keyExtractor):
        # type: (ToIntFunction) -> Comparator
        pass

    def thenComparingLong(self, keyExtractor):
        # type: (ToLongFunction) -> Comparator
        pass


class Iterator(object):
    def forEachRemaining(self, action):
        # type: (Consumer) -> None
        pass

    def hasNext(self):
        # type: () -> bool
        raise NotImplementedError

    def next(self):
        # type: () -> E
        raise NotImplementedError

    def remove(self):
        # type: () -> bool
        pass


class Spliterator(object):
    CONCURRENT = None  # type: int
    DISTINCT = None  # type: int
    IMMUTABLE = None  # type: int
    NONNULL = None  # type: int
    ORDERED = None  # type: int
    SIZED = None  # type: int
    SORTED = None  # type: int
    SUBSIZED = None  # type: int

    def characteristics(self):
        # type: () -> int
        raise NotImplementedError

    def estimateSize(self):
        # type: () -> long
        raise NotImplementedError

    def forEachRemaining(self, action):
        # type: (Consumer) -> None
        pass

    def getComparator(self):
        # type: () -> Comparator
        pass

    def getExactSizeIfKnown(self):
        # type: () -> long
        pass

    def hasCharacteristics(self, characteristics):
        # type: (int) -> bool
        pass

    def tryAdvance(self, action):
        # type: (Consumer) -> bool
        raise NotImplementedError

    def trySplit(self):
        # type: () -> Spliterator
        raise NotImplementedError


class Calendar(Object):
    ALL_STYLES = None  # type: int
    AM = None  # type: int
    AM_PM = None  # type: int
    APRIL = None  # type: int
    AUGUST = None  # type: int
    DATE = None  # type: int
    DAY_OF_MONTH = None  # type: int
    DAY_OF_WEEK = None  # type: int
    DAY_OF_WEEK_IN_MONTH = None  # type: int
    DAY_OF_YEAR = None  # type: int
    DECEMBER = None  # type: int
    DST_OFFSET = None  # type: int
    ERA = None  # type: int
    FEBRUARY = None  # type: int
    FIELD_COUNT = None  # type: int
    FRIDAY = None  # type: int
    HOUR = None  # type: int
    HOUR_OF_DAY = None  # type: int
    JANUARY = None  # type: int
    JULY = None  # type: int
    JUNE = None  # type: int
    LONG = None  # type: int
    LONG_FORMAT = None  # type: int
    LONG_STANDALONE = None  # type: int
    MARCH = None  # type: int
    MAY = None  # type: int
    MILLISECOND = None  # type: int
    MINUTE = None  # type: int
    MONDAY = None  # type: int
    MONTH = None  # type: int
    NARROW_FORMAT = None  # type: int
    NARROW_STANDALONE = None  # type: int
    NOVEMBER = None  # type: int
    OCTOBER = None  # type: int
    PM = None  # type: int
    SATURDAY = None  # type: int
    SECOND = None  # type: int
    SEPTEMBER = None  # type: int
    SHORT = None  # type: int
    SHORT_FORMAT = None  # type: int
    SHORT_STANDALONE = None  # type: int
    SUNDAY = None  # type: int
    THURSDAY = None  # type: int
    TUESDAY = None  # type: int
    UNDECIMBER = None  # type: int
    WEDNESDAY = None  # type: int
    WEEK_OF_MONTH = None  # type: int
    WEEK_OF_YEAR = None  # type: int
    YEAR = None  # type: int
    ZONE_OFFSET = None  # type: int

    def add(self, field, amount):
        # type: (int, int) -> None
        raise NotImplementedError

    def after(self, when):
        # type: (Object) -> bool
        pass

    def before(self, when):
        # type: (Object) -> bool
        pass

    def clear(self, field=None):
        # type: (Optional[int]) -> None
        pass

    def clone(self):
        # type: () -> Object
        pass

    def compareTo(self, anotherCalendar):
        # type: (Calendar) -> int
        pass

    def get(self, field):
        # type: (int) -> int
        pass

    def getActualMaximum(self, field):
        # type: (int) -> int
        pass

    def getActualMinimum(self, field):
        # type: (int) -> int
        pass

    @staticmethod
    def getAvailableCalendarTypes():
        # type: () -> Set[String]
        pass

    @staticmethod
    def getAvailableLocales():
        # type: () -> List[Locale]
        pass

    def getCalendarType(self):
        # type: () -> String
        pass

    def getDisplayName(self, field, style, locale):
        # type: (int, int, Locale) -> String
        pass

    def getDisplayNames(self, field, style, locale):
        # type: (int, int, Locale) -> Dict[String, int]
        pass

    def getFirstDayOfWeek(self):
        # type: () -> int
        pass

    def getGreatestMinimum(self, field):
        # type: (int) -> int
        raise NotImplementedError

    def getInstance(self, *args):
        # type: (*Any) -> Calendar
        pass

    def getTimeZone(self):
        # type: () -> TimeZone
        pass

    def getWeeksInWeekYear(self):
        # type: () -> int
        pass

    def getWeekYear(self):
        # type: () -> int
        pass

    def isLenient(self):
        # type: () -> bool
        pass

    def isSet(self, field):
        # type: (int) -> bool
        pass

    def isWeekDateSUpported(self):
        # type: () -> bool
        pass

    def roll(self, field, amount):
        # type: (int, int) -> None
        pass

    def set(self, *args):
        # type: (*int) -> None
        pass

    def setFirstDayOfWeek(self, value):
        # type: (int) -> None
        pass

    def setLenient(self, lenient):
        # type: (bool) -> None
        pass

    def setMinimalDaysInFirstWeek(self, value):
        # type: (int) -> None
        pass

    def setTime(self, date):
        # type: (Date) -> None
        pass

    def setTimeInMillis(self, millis):
        # type: (long) -> None
        pass

    def setTimeZone(self, value):
        # type: (TimeZone) -> None
        pass

    def setWeekDate(self, weekYear, weekOfYear, dayOfWeek):
        # type: (int, int, int) -> None
        pass

    def toInstant(self):
        # type: () -> Instant
        pass


class Currency(Object):
    @staticmethod
    def getAvailableCurrencies():
        # type: () -> Set[Currency]
        pass

    def getCurrencyCode(self):
        # type: () -> String
        pass

    def getDisplayName(self, locale=None):
        # type: (Optional[Locale]) -> String
        pass

    @staticmethod
    def getInstance(arg):
        # type: (Union[Locale, String]) -> Currency
        pass

    def getNumericCode(self):
        # type: () -> int
        pass

    def getNumericCodeAsString(self):
        # type: () -> String
        pass

    def getSymbol(self, locale=None):
        # type: (Optional[Locale]) -> String
        pass


class Date(Object):
    """The class Date represents a specific instant in time, with
    millisecond precision.
    """

    def __init__(self, date=None):
        # type: (Optional[long]) -> None
        print(self, date)

    def after(self, when):
        # type: (Date) -> bool
        pass

    def before(self, when):
        # type: (Date) -> bool
        pass

    def compareTo(self, anotherDate):
        # type: (Date) -> int
        pass

    def getTime(self):
        # type: () -> long
        pass

    def setTime(self, time):
        # type: (long) -> None
        pass


class EventObject(Object):
    """The root class from which all event state objects shall be
    derived.

    All Events are constructed with a reference to the object, the
    "source", that is logically deemed to be the object upon which the
    Event in question initially occurred upon.
    """

    _source = None  # type: Object

    def __init__(self, source):
        # type: (Object) -> None
        self._source = source

    def getSource(self):
        # type: () -> Object
        return self._source


class Locale(Object):
    """A Locale object represents a specific geographical, political, or
    cultural region. An operation that requires a Locale to perform its
    task is called locale-sensitive and uses the Locale to tailor
    information for the user. For example, displaying a number is a
    locale-sensitive operation; the number should be formatted according
    to the customs and conventions of the user's native country, region,
    or culture.
    """

    country = None  # type: Optional[str]
    language = None  # type: str
    variant = None  # type: Optional[str]

    def __init__(self, language, country=None, variant=None):
        # type: (str, Optional[str], Optional[str]) -> None
        self.language = language
        self.country = country
        self.variant = variant

    def __repr__(self):
        return "{!r}".format(self.__str__())

    def __str__(self):
        ret = self.language
        if self.country:
            ret += "_{}".format(self.country)
        if self.variant:
            ret += "_{}".format(self.variant)
        return unicode(ret)

    @classproperty
    def CANADA(self):
        # type: () -> Locale
        return Locale("en", "CA")

    @classproperty
    def CANADA_FRENCH(self):
        # type: () -> Locale
        return Locale("fr", "CA")

    @classproperty
    def CHINA(self):
        # type: () -> Locale
        return Locale("zh", "CN")

    @classproperty
    def CHINESE(self):
        # type: () -> Locale
        return Locale("zh")

    @classproperty
    def ENGLISH(self):
        # type: () -> Locale
        return Locale("en")

    @classproperty
    def FRANCE(self):
        # type: () -> Locale
        return Locale("fr", "FR")

    @classproperty
    def FRENCH(self):
        # type: () -> Locale
        return Locale("fr")

    @classproperty
    def GERMAN(self):
        # type: () -> Locale
        return Locale("de")

    @classproperty
    def GERMANY(self):
        # type: () -> Locale
        return Locale("de", "DE")

    @classproperty
    def ITALIAN(self):
        # type: () -> Locale
        return Locale("it")

    @classproperty
    def ITALY(self):
        # type: () -> Locale
        return Locale("it", "IT")

    @classproperty
    def JAPAN(self):
        # type: () -> Locale
        return Locale("ja", "JP")

    @classproperty
    def JAPANESE(self):
        # type: () -> Locale
        return Locale("ja")

    @classproperty
    def KOREA(self):
        # type: () -> Locale
        return Locale("ko", "KR")

    @classproperty
    def KOREAN(self):
        # type: () -> Locale
        return Locale("ko")

    @classproperty
    def PRC(self):
        # type: () -> Locale
        return Locale("zh", "CN")

    @classproperty
    def SIMPLIFIED_CHINESE(self):
        # type: () -> Locale
        return Locale("zh", "CN")

    @classproperty
    def TAIWAN(self):
        # type: () -> Locale
        return Locale("zh", "TW")

    @classproperty
    def TRADITIONAL_CHINESE(self):
        # type: () -> Locale
        return Locale("zh", "TW")

    @classproperty
    def UK(self):
        # type: () -> Locale
        return Locale("en", "GB")

    @classproperty
    def US(self):
        # type: () -> Locale
        return Locale("en", "US")


class TimeZone(Object):
    LONG = None  # type: int
    SHORT = None  # type: int

    def clone(self):
        # type: () -> Object
        pass

    @staticmethod
    def getAvailableIDs(rawOffset=None):
        # type: (Optional[int]) -> List[String]
        pass

    @staticmethod
    def getDefault():
        # type: () -> TimeZone
        pass

    def getDisplayName(self, *args):
        # type: (*Any) -> String
        pass

    def getDSTSavings(self):
        # type: () -> int
        pass

    def getID(self):
        # type: () -> String
        pass

    def getOffset(self, *args):
        # type: (*Any) -> int
        pass

    def getRawOffset(self):
        # type: () -> int
        raise NotImplementedError

    @staticmethod
    def getTimeZone(arg):
        # type: (Union[String, ZoneId]) -> TimeZone
        pass

    def hasSameRules(self, other):
        # type: (TimeZone) -> bool
        pass

    def isDaylightTime(self, date):
        # type: (Date) -> bool
        raise NotImplementedError

    def observesDaylightTime(self):
        # type: () -> bool
        pass

    @staticmethod
    def setDefault(zone):
        # type: (TimeZone) -> None
        pass

    def setID(self, ID):
        # type: (String) -> None
        pass

    def setRawOffset(self, offsetMillis):
        # type: (int) -> None
        raise NotImplementedError

    def toZoneId(self):
        # type: () -> ZoneId
        pass

    def useDaylightTime(self):
        # type: () -> bool
        raise NotImplementedError


class UUID(Object):
    def __init__(self, mostSigBits, leastSigBits):
        # type: (long, long) -> None
        self._leastSigbits = leastSigBits
        self._mostSigBits = mostSigBits

    def clockSequence(self):
        # type: () -> int
        pass

    def compareTo(self, val):
        # type: (UUID) -> int
        pass

    @staticmethod
    def fromString(name):
        # type: (String) -> UUID
        pass

    def getLeastSignificantBits(self):
        # type: () -> long
        return self._leastSigbits

    def getMostSignificantBits(self):
        # type: () -> long
        return self._mostSigBits

    @staticmethod
    def nameUUIDFromBytes(name):
        # type: (bytearray) -> UUID
        pass

    def node(self):
        # type: () -> long
        pass

    @staticmethod
    def randomUUID():
        # type: () -> UUID
        pass

    def timestamp(self):
        # type: () -> long
        pass

    def variant(self):
        # type: () -> int
        pass

    def version(self):
        # type: () -> int
        pass



from java.lang import Object


class Animator(Object):
    def cancel(self):
        # type: () -> bool
        pass

    def pause(self):
        # type: () -> None
        pass

    def resume(self):
        # type: () -> None
        pass



from __future__ import print_function

__all__ = [
    "AttributedCharacterIterator",
    "CharacterIterator",
    "DateFormat",
    "DateFormatSymbols",
    "FieldPosition",
    "Format",
    "NumberFormat",
    "ParsePosition",
    "SimpleDateFormat",
]

from typing import Any, Dict, List, Optional, Set, Union

from java.lang import Number, Object, String, StringBuffer
from java.math import RoundingMode
from java.util import Calendar, Currency, Date, Locale, TimeZone


class CharacterIterator(object):
    DONE = None  # type: str

    def clone(self):
        # type: () -> str
        raise NotImplementedError

    def current(self):
        # type: () -> str
        raise NotImplementedError

    def first(self):
        # type: () -> str
        raise NotImplementedError

    def getBeginIndex(self):
        # type: () -> int
        raise NotImplementedError

    def getEndIndex(self):
        # type: () -> int
        raise NotImplementedError

    def getIndex(self):
        # type: () -> int
        raise NotImplementedError

    def last(self):
        # type: () -> str
        raise NotImplementedError

    def next(self):
        # type: () -> str
        raise NotImplementedError

    def previous(self):
        # type: () -> str
        raise NotImplementedError

    def setIndex(self, position):
        # type: (int) -> str
        raise NotImplementedError


class AttributedCharacterIterator(CharacterIterator):
    def clone(self):
        # type: () -> str
        raise NotImplementedError

    def current(self):
        # type: () -> str
        raise NotImplementedError

    def first(self):
        # type: () -> str
        raise NotImplementedError

    def getAllAttributeKeys(self):
        # type: () -> Set[AttributedCharacterIterator.Attribute]
        raise NotImplementedError

    def getAttribute(self, attribute):
        # type: (AttributedCharacterIterator.Attribute) -> Object
        raise NotImplementedError

    def getAttributes(self):
        # type: () -> Dict[AttributedCharacterIterator.Attribute, Object]  # noqa: W505
        raise NotImplementedError

    def getBeginIndex(self):
        # type: () -> int
        raise NotImplementedError

    def getEndIndex(self):
        # type: () -> int
        raise NotImplementedError

    def getIndex(self):
        # type: () -> int
        raise NotImplementedError

    def last(self):
        # type: () -> str
        raise NotImplementedError

    def next(self):
        # type: () -> str
        raise NotImplementedError

    def previous(self):
        # type: () -> str
        raise NotImplementedError

    def setIndex(self, position):
        # type: (int) -> str
        raise NotImplementedError

    class Attribute(Object):
        INPUT_METHOD_SEGMENT = None  # type: AttributedCharacterIterator.Attribute
        LANGUAGE = None  # type: AttributedCharacterIterator.Attribute
        READING = None  # type: AttributedCharacterIterator.Attribute


class FieldPosition(Object):
    def __init__(self, *args):
        # type: (*Any) -> None
        pass

    def getBeginIndex(self):
        # type: () -> int
        pass

    def getEndIndex(self):
        # type: () -> int
        pass

    def getField(self):
        # type: () -> Any
        pass

    def getFieldAttribute(self):
        # type: () -> Format.Field
        pass

    def setBeginIndex(self, bi):
        # type: (int) -> None
        pass

    def setEndIndex(self, ei):
        # type: (int) -> None
        pass


class Format(Object):
    def clone(self):
        # type: () -> Object
        pass

    def formatToCharacterIterator(self, obj):
        # type: (Object) -> AttributedCharacterIterator
        pass

    def parseObject(self, source, pos=None):
        # type: (String, Optional[ParsePosition]) -> Object
        pass

    class Field(AttributedCharacterIterator.Attribute):
        pass


class DateFormat(Format):
    AM_PM_FIELD = None  # type: int
    DATE_FIELD = None  # type: int
    DAY_OF_WEEK_FIELD = None  # type: int
    DAY_OF_WEEK_IN_MONTH_FIELD = None  # type: int
    DAY_OF_YEAR_FIELD = None  # type: int
    DEFAULT = None  # type: int
    ERA_FIELD = None  # type: int
    FULL = None  # type: int
    HOUR_OF_DAY0_FIELD = None  # type: int
    HOUR_OF_DAY1_FIELD = None  # type: int
    HOUR0_FIELD = None  # type: int
    HOUR1_FIELD = None  # type: int
    LONG = None  # type: int
    MEDIUM = None  # type: int
    MILLISECOND_FIELD = None  # type: int
    MONTH_FIELD = None  # type: int
    SECOND_FIELD = None  # type: int
    SHORT = None  # type: int
    TIMEZONE_FIELD = None  # type: int
    WEEK_OF_MONTH_FIELD = None  # type: int
    WEEK_OF_YEAR_FIELD = None  # type: int
    YEAR_FIELD = None  # type: int

    def format(
        self,
        date,  # type: Union[Date, Object]
        toAppendTo=None,  # type: Optional[StringBuffer]
        pos=None,  # type: Optional[FieldPosition]
    ):  # type: (...) -> Union[String, StringBuffer]
        pass

    @staticmethod
    def getAvailableLocales():
        # type: () -> List[Locale]
        pass

    def getCalendar(self):
        # type: () -> Calendar
        pass

    @staticmethod
    def getDateInstance(style=None, aLocale=None):
        # type: (Optional[int], Optional[Locale]) -> DateFormat
        pass

    @staticmethod
    def getDateTimeInstance():
        # type: () -> DateFormat
        pass

    @staticmethod
    def getInstance():
        # type: () -> DateFormat
        pass

    def getNumberFormat(self):
        # type: () -> NumberFormat
        pass

    @staticmethod
    def getTimeInstance(style, aLocale=None):
        # type: (int, Optional[Locale]) -> DateFormat
        pass

    def getTimeZone(self):
        # type: () -> TimeZone
        pass

    def isLenient(self):
        # type: () -> bool
        pass

    def parse(self, source):
        # type: (String) -> Date
        pass

    def setCalendar(self, newCalendar):
        # type: (Calendar) -> None
        pass

    def setLenient(self, lenient):
        # type: (bool) -> None
        pass

    def setNumberFormat(self, newNumberFormat):
        # type: (NumberFormat) -> None
        pass

    def setTimeZone(self, zone):
        # type: (TimeZone) -> None
        pass


class DateFormatSymbols(Object):
    def __init__(self, locale=None):
        # type: (Optional[Locale]) -> None
        print(locale)

    def clone(self):
        # type: () -> Object
        pass

    def getAmPmStrings(self):
        # type: () -> List[String]
        pass

    @staticmethod
    def getAvailableLocales():
        # type: () -> List[Locale]
        pass

    def getEras(self):
        # type: () -> List[String]
        pass

    @staticmethod
    def getInstance(locale=None):
        # type: (Optional[Locale]) -> DateFormatSymbols
        pass

    def getLocalPatternChars(self):
        # type: () -> String
        pass

    def getMonths(self):
        # type: () -> List[String]
        pass

    def getShortMonths(self):
        # type: () -> List[String]
        pass

    def getShortWeekdays(self):
        # type: () -> List[String]
        pass

    def getWeekdays(self):
        # type: () -> List[String]
        pass

    def getZoneStrings(self):
        # type: () -> List[List[String]]
        pass

    def setAmPmStrings(self, newAmpms):
        # type: (List[String]) -> None
        pass

    def setEras(self, newEras):
        # type: (List[String]) -> None
        pass

    def setLocalPatternChars(self, newLocalPatternChars):
        # type: (String) -> None
        pass

    def setMonths(self, newMonths):
        # type: (List[String]) -> None
        pass

    def setShortMonths(self, newShortMonths):
        # type: (List[String]) -> None
        pass

    def setShortWeekdays(self, newShortWeekdays):
        # type: (List[String]) -> None
        pass

    def setWeekdays(self, newWeekdays):
        # type: (List[String]) -> None
        pass

    def setZoneStrings(self, newZoneStrings):
        # type: (List[List[String]]) -> None
        pass


class NumberFormat(Format):
    FRACTION_FIELD = None  # type: int
    INTEGER_FIELD = None  # type: int

    def format(
        self,
        number,  # type: Union[float, long, Object]
        toAppendTo=None,  # type: Optional[StringBuffer]
        pos=None,  # type: Optional[FieldPosition]
    ):
        # type: (...) -> Union[String, StringBuffer]
        raise NotImplementedError

    @staticmethod
    def getAvailableLocales():
        # type: () -> List[Locale]
        pass

    def getCurrency(self):
        # type: () -> Currency
        pass

    @staticmethod
    def getCurrencyInstance(inLocale=None):
        # type: (Optional[Locale]) -> NumberFormat
        pass

    @staticmethod
    def getInstance(inLocale=None):
        # type: (Optional[Locale]) -> NumberFormat
        pass

    @staticmethod
    def getIntegerInstance(inLocale=None):
        # type: (Optional[Locale]) -> NumberFormat
        pass

    def getMaximumFractionDigits(self):
        # type: () -> int
        pass

    def getMaximumIntegerDigits(self):
        # type: () -> int
        pass

    def getMinimumFractionDigits(self):
        # type: () -> int
        pass

    def getMinimumIntegerDigits(self):
        # type: () -> int
        pass

    @staticmethod
    def getNumberInstance(inLocale=None):
        # type: (Optional[Locale]) -> NumberFormat
        pass

    @staticmethod
    def getPercentInstance(inLocale=None):
        # type: (Optional[Locale]) -> NumberFormat
        pass

    def getRoundingMode(self):
        # type: () -> RoundingMode
        pass

    def isGroupingUsed(self):
        # type: () -> bool
        pass

    def isParseIntegerOnly(self):
        # type: () -> bool
        pass

    def parse(self, source):
        # type: (String) -> Number
        pass

    def setCurrency(self, currency):
        # type: (Currency) -> None
        pass

    def setGroupingUsed(self, newValue):
        # type: (bool) -> None
        pass

    def setMaximumFractionDigits(self, newValue):
        # type: (int) -> None
        pass

    def setMaximumIntegerDigits(self, newValue):
        # type: (int) -> None
        pass

    def setMinimumFractionDigits(self, newValue):
        # type: (int) -> None
        pass

    def setMinimumIntegerDigits(self, newValue):
        # type: (int) -> None
        pass

    def setParseIntegerOnly(self, value):
        # type: (bool) -> None
        pass

    def setRoundingMode(self, roundingMode):
        # type: (RoundingMode) -> None
        pass

    class Field(Format.Field):
        CURRENCY = None  # type: NumberFormat.Field
        DECIMAL_SEPARATOR = None  # type: NumberFormat.Field
        EXPONENT = None  # type: NumberFormat.Field
        EXPONENT_SIGN = None  # type: NumberFormat.Field
        EXPONENT_SYMBOL = None  # type: NumberFormat.Field
        FRACTION = None  # type: NumberFormat.Field
        GROUPING_SEPARATOR = None  # type: NumberFormat.Field
        INTEGER = None  # type: NumberFormat.Field
        PERCENT = None  # type: NumberFormat.Field
        PERMILLE = None  # type: NumberFormat.Field
        SIGN = None  # type: NumberFormat.Field


class ParsePosition(Object):
    def __init__(self, index):
        # type: (int) -> None
        print(index)

    def getErrorIndex(self):
        # type: () -> int
        pass

    def getIndex(self):
        # type: () -> int
        pass

    def setErrorIndex(self, ei):
        # type: (int) -> None
        pass

    def setIndex(self, index):
        # type: (int) -> None
        pass


class SimpleDateFormat(DateFormat):
    def __init__(
        self,
        pattern=None,  # type: Optional[String]
        arg=None,  # type: Union[DateFormatSymbols, Locale, None]
    ):
        # type: (...) -> None
        print(pattern, arg)

    def applyLocalizedPattern(self, pattern):
        # type: (String) -> None
        pass

    def applyPattern(self, pattern):
        # type: (String) -> None
        pass

    def get2DigitYearStart(self):
        # type: () -> Date
        pass

    def getDateFormatSymbols(self):
        # type: () -> DateFormatSymbols
        pass

    def set2DigitYearStart(self, startDate):
        # type: (Date) -> None
        pass

    def setDateFormatSymbols(self, newFormatSymbols):
        # type: (DateFormatSymbols) -> None
        pass

    def toLocalizedPattern(self):
        # type: () -> String
        pass

    def toPattern(self):
        # type: () -> String
        pass



__all__ = ["TimeUnit"]

from typing import Any, List

from java.lang import Enum, Object, Thread
from java.time.temporal import ChronoUnit


class TimeUnit(Enum):
    DAYS = None  # type: TimeUnit
    HOURS = None  # type: TimeUnit
    MICROSECONDS = None  # type: TimeUnit
    MILLISECONDS = None  # type: TimeUnit
    MINUTES = None  # type: TimeUnit
    NANOSECONDS = None  # type: TimeUnit
    SECONDS = None  # type: TimeUnit

    def convert(self, *args):
        # type: (*Any) -> long
        pass

    @staticmethod
    def of(chronoUnit):
        # type: (ChronoUnit) -> TimeUnit
        pass

    def sleep(self, timeout):
        # type: (long) -> None
        pass

    def timedJoin(self, thread, timeout):
        # type: (Thread, long) -> None
        pass

    def timedWait(self, obj, timeout):
        # type: (Object, long) -> None
        pass

    def toChronoUnit(self):
        # type: () -> ChronoUnit
        pass

    def toDays(self, duration):
        # type: (long) -> long
        pass

    def toHours(self, duration):
        # type: (long) -> long
        pass

    def toMicros(self, duration):
        # type: (long) -> long
        pass

    def toMillis(self, duration):
        # type: (long) -> long
        pass

    def toMinutes(self, duration):
        # type: (long) -> long
        pass

    def toNanos(self, duration):
        # type: (long) -> long
        pass

    def toSeconds(self, duration):
        # type: (long) -> long
        pass

    @staticmethod
    def values():
        # type: () -> List[TimeUnit]
        pass



from __future__ import print_function

__all__ = [
    "AbstractDataset",
    "BasicDataset",
    "Dataset",
    "JsonPath",
    "Path",
    "QualifiedPath",
    "TypeUtilities",
]

from typing import Any, List, Optional, Set, TypeVar, Union

from com.inductiveautomation.ignition.common.document import DocumentElement
from com.inductiveautomation.ignition.common.gson import Gson, JsonElement
from com.inductiveautomation.ignition.common.model.values import (
    QualifiedValue,
    QualityCode,
)
from com.inductiveautomation.ignition.common.sqltags.model.types import (
    DataQuality,
    DataType,
)
from java.awt import Color
from java.lang import Class, Number, Object, String
from java.util import UUID, Comparator, Date, Locale
from org.json import JSONObject
from org.python.core import PyObject

T = TypeVar("T")


class Dataset(object):
    """A dataset is a collection of values arranged in a structured
    format.

    Most datasets are two-dimensional -- they can be viewed as a table
    with rows and columns being the two dimensions. Values in a dataset
    are usually accessed by specifying one index for each dimension of
    data (row and column for tables).
    """

    def binarySearch(self, column, key):
        # type: (int, Any) -> int
        pass

    def getColumnAsList(self, col):
        # type: (int) -> List[Any]
        """Returns the column at the specified index as a list.

        Args:
            col: The column index. Zero-indexed.

        Returns:
            The column at the specified index as a list.
        """
        pass

    def getColumnCount(self):
        # type: () -> int
        """Returns the number of columns in the dataset.

        Returns:
            The number of columns in the dataset.
        """
        raise NotImplementedError

    def getColumnIndex(self, colName):
        # type: (String) -> int
        """Returns the index of the column with the name colName.

        Args:
            colName: The name of the column.

        Returns:
            The index of the column with the name colName.
        """
        raise NotImplementedError

    def getColumnName(self, col):
        # type: (int) -> String
        """Returns the name of the column at the index colIndex.

        Args:
            col: The column number. Zero-indexed.

        Returns:
            The name of the column at the index colIndex.
        """
        raise NotImplementedError

    def getColumnNames(self):
        # type: () -> List[String]
        """Returns a list with the names of all the columns.

        Returns:
            A list with the names of all the columns.
        """
        raise NotImplementedError

    def getColumnType(self, col):
        # type: (int) -> Class
        """Returns the type of the column at the index.

        Args:
            col: The column number. Zero-indexed.

        Returns:
            The type of the column at the index.
        """
        raise NotImplementedError

    def getColumnTypes(self):
        # type: () -> List[Class]
        """Returns a list with the types of all the columns.

        Returns:
            A list with the types of all the columns.
        """
        raise NotImplementedError

    def getPrimitiveValueAt(self, row, col):
        # type: (int, int) -> float
        """If the given column is a numeric type or a Date, then the
        value will be returned as a double.

        Args:
            row: The row index. Zero-based index.
            col: The column index. Zero-based index.

        Raises:
            IllegalArgumentException: if the value at row, col is not a
                number or Date.
            UnsupportedOperationException: If the Dataset implementation
                declines to implement this operation.
        """
        raise NotImplementedError

    def getQualityAt(self, row, col):
        # type: (int, int) -> QualityCode
        """Returns the quality of the value at the given location.

        Args:
            row: The row index. Zero-based index.
            col: The column index. Zero-based index.

        Raises:
            ArrayIndexOutOfBoundsException: If the given row, col is out
                of range and hasQualityData() returns true.
        """
        raise NotImplementedError

    def getRowCount(self):
        # type: () -> int
        """Returns the number of rows in the dataset.

        Returns:
            The number of rows in the dataset.
        """
        raise NotImplementedError

    def getValueAt(self, row, col):
        # type: (int, Union[int, String]) -> Any
        """Returns the value at the specified row index and column name
        or index.

        Args:
            row: The row number. Zero-indexed.
            col: The column number (zero-indexed) or name.

        Returns:
            The value found at the row and column.
        """
        raise NotImplementedError

    def hasQualityData(self):
        # type: () -> bool
        """Whether this dataset has any quality data to report.

        Returns:
            Whether this dataset has any quality data to report.
        """
        pass


class AbstractDataset(Dataset):
    _columnNames = None  # type: List[String]
    _columnNamesLowercase = None  # type: List[String]
    _columnTypes = None  # type: List[Class]
    _qualityCodes = None  # type: Optional[List[List[QualityCode]]]

    def __init__(
        self,
        columnNames,  # type: List[String]
        columnTypes,  # type: List[Class]
        qualityCodes=None,  # type: Optional[List[List[QualityCode]]]
    ):
        # type: (...) -> None
        self._columnNames = columnNames
        self._columnTypes = columnTypes
        self._qualityCodes = qualityCodes

    @staticmethod
    def convertToQualityCodes(dataQualities):
        # type: (List[List[DataQuality]]) -> List[List[QualityCode]]
        pass

    def getBulkQualityCodes(self):
        # type: () -> List[List[QualityCode]]
        pass

    def getColumnCount(self):
        # type: () -> int
        """Returns the number of columns in the dataset.

        Returns:
            The number of columns in the dataset.
        """
        pass

    def getColumnIndex(self, colName):
        # type: (String) -> int
        """Returns the index of the column with the name colName.

        Args:
            colName: The name of the column.

        Returns:
            The index of the column with the name colName.
        """
        pass

    def getColumnName(self, col):
        # type: (int) -> String
        """Returns the name of the column at the index colIndex.

        Args:
            col: The column number. Zero-indexed.

        Returns:
            The name of the column at the index colIndex.
        """
        pass

    def getColumnNames(self):
        # type: () -> List[String]
        """Returns a list with the names of all the columns.

        Returns:
            A list with the names of all the columns.
        """
        pass

    def getColumnType(self, col):
        # type: (int) -> Class
        """Returns the type of the column at the index.

        Args:
            col: The column number. Zero-indexed.

        Returns:
            The type of the column at the index.
        """
        pass

    def getColumnTypes(self):
        # type: () -> List[Class]
        """Returns a list with the types of all the columns.

        Returns:
            A list with the types of all the columns.
        """
        pass

    def getPrimitiveValueAt(self, row, col):
        # type: (int, int) -> float
        """If the given column is a numeric type or a Date, then the
        value will be returned as a double.

        Args:
            row: The row index. Zero-based index.
            col: The column index. Zero-based index.

        Raises:
            IllegalArgumentException: if the value at row, col is not a
                number or Date.
            UnsupportedOperationException: If the Dataset implementation
                declines to implement this operation.
        """
        pass

    def getQualityAt(self, row, col):
        # type: (int, int) -> QualityCode
        """Returns the quality of the value at the given location.

        Args:
            row: The row index. Zero-based index.
            col: The column index. Zero-based index.

        Raises:
            ArrayIndexOutOfBoundsException: If the given row, col is out
                of range and hasQualityData() returns true.
        """
        pass

    def getRowCount(self):
        # type: () -> int
        """Returns the number of rows in the dataset.

        Returns:
            The number of rows in the dataset.
        """
        pass

    def getValueAt(self, row, col):
        # type: (int, Union[int, String]) -> Any
        """Returns the value at the specified row index and column name
        or index.

        Args:
            row: The row number. Zero-indexed.
            col: The column number (zero-indexed) or name.

        Returns:
            The value found at the row and column.
        """
        pass

    def setColumnNames(self, arg):
        # type: (List[str]) -> None
        pass

    def setColumnTypes(self, arg):
        # type: (List[Class]) -> None
        pass

    def setDirty(self):
        # type: () -> None
        pass


class BasicDataset(AbstractDataset):
    def __init__(self, *args):
        # type: (*Any) -> None
        print(args)
        super(BasicDataset, self).__init__([""], [Class()])

    def columnContainsNulls(self, col):
        # type: (int) -> bool
        pass

    def datasetContainsNulls(self):
        # type: () -> bool
        pass

    def getData(self):
        # type: () -> Any
        pass

    def setAllDirectly(self, columnNames, columnTypes, data):
        # type: (List[String], List[Class], Any) -> None
        pass

    def setDataDirectly(self, arg):
        # type: (Any) -> None
        pass

    def setFromXML(self, columnNames, columnTypes, encodedData, rowCount):
        # type: (List[String], List[Class], String, int) -> None
        pass

    def setValueAt(self, row, col, value):
        # type: (int, int, Any) -> None
        pass


class JsonPath(Object):
    ROOT = None  # type: JsonPath

    def createChildPath(self, arg):
        # type: (Union[JsonPath, int, String]) -> JsonPath
        pass

    def getAsLinkedList(self):
        # type: () -> Any
        pass

    def getDepth(self):
        # type: () -> int
        pass

    def getKey(self):
        # type: () -> Object
        pass

    def getParent(self):
        # type: () -> JsonPath
        pass

    def getPathElements(self):
        # type: () -> List[JsonPath]
        pass

    def getSubPath(self, startingDepth):
        # type: (int) -> JsonPath
        pass

    def isAncestorOf(self, element):
        # type: (JsonPath) -> bool
        pass

    def isRelatedTo(self, other):
        # type: (JsonPath) -> bool
        pass

    def isRoot(self):
        # type: () -> bool
        pass

    @staticmethod
    def isValidIdentifier(test):
        # type: (String) -> bool
        pass

    @staticmethod
    def parse(path):
        # type: (String) -> String
        pass


class Path(object):
    SERIALIZATION_WHITELIST = None  # type: Set[Class]

    def getLastPathComponent(self):
        # type: () -> String
        raise NotImplementedError

    def getParentPath(self):
        # type: () -> Path
        raise NotImplementedError

    def getPathComponent(self, i):
        # type: (int) -> String
        raise NotImplementedError

    def getPathLength(self):
        # type: () -> int
        raise NotImplementedError

    def isAncestorOf(self, path):
        # type: (Path) -> bool
        raise NotImplementedError


class QualifiedPath(Object):
    def extend(self, id_, value):
        pass

    def getFirstPathComponent(self):
        pass

    def getFirstPathComponentId(self):
        pass


class TypeUtilities(Object):
    DATE_FORMAT_STRING = "yyyyMMdd.HHmmssSSSZ"  # type: String
    NULL_SAFE_CASE_INSENSITIVE_ORDER = None  # type: Comparator

    @staticmethod
    def anyEqual(value, *args):
        # type: (T, T) -> bool
        pass

    @staticmethod
    def coerce(value, destType):
        # type: (Object, Class) -> Object
        pass

    @staticmethod
    def coerceForLocale(value, target, valueLocale):
        # type: (Object, Class, Locale) -> Object
        pass

    @staticmethod
    def coerceGeneric(value, destType):
        # type: (Object, Class) -> T
        pass

    @staticmethod
    def coerceLocaleSafe(str, type):
        # type: (String, Class) -> Object
        pass

    @staticmethod
    def coerceNullSafe(value, destType):
        # type: (Object, Class) -> Object
        pass

    @staticmethod
    def coerceNumberForLocale(value, destType, locale):
        # type: (Object, Class, Locale) -> Object
        pass

    @staticmethod
    def coerceNumberNullSafe(value, destType, locale):
        # type: (Object, Class, Locale) -> Object
        pass

    @staticmethod
    def colorToHex(c):
        # type: (Color) -> String
        pass

    @staticmethod
    def compareInts(foo, bar):
        # type: (int, int) -> int
        pass

    @staticmethod
    def compareNullHigh(c1, c2):
        # type: (T, T) -> T
        pass

    @staticmethod
    def compareNullLow(c1, c2):
        # type: (T, T) -> T
        pass

    @staticmethod
    def datasetFromJSON(json):
        # type: (JSONObject) -> Dataset
        pass

    @staticmethod
    def datasetFromJsonString(jsonStr):
        # type: (String) -> Dataset
        pass

    @staticmethod
    def datasetToGson(data, gson=None):
        # type: (Dataset, Optional[Gson]) -> JsonElement
        pass

    @staticmethod
    def datasetToJSON(data):
        # type: (Dataset) -> JSONObject
        pass

    @staticmethod
    def deepEquals(o1, o2, checkArrayTypes):
        # type: (Object, Object, bool) -> bool
        pass

    @staticmethod
    def equalsIgnoreCase(o1, o2):
        # type: (Object, Object) -> bool
        pass

    @staticmethod
    def fromString(value, dest, locale):
        # type: (String, Class, Locale) -> Object
        pass

    @staticmethod
    def getColorFromString(color):
        # type: (String) -> Color
        pass

    @staticmethod
    def getFirstOrNull(list_):
        # type: (List[T]) -> T
        pass

    @staticmethod
    def getInitValueForClass(c):
        # type: (Class) -> Object
        pass

    @staticmethod
    def getLastNameComponent(name):
        # type: (String) -> String
        pass

    @staticmethod
    def getPrimitiveType(c):
        # type: (Class) -> Object
        pass

    @staticmethod
    def getWrapperType(c):
        # type: (Class) -> Class
        pass

    @staticmethod
    def gsonToPy(element):
        # type: (JsonElement) -> PyObject
        pass

    @staticmethod
    def hasPrimitiveType(c):
        # type: (Class) -> bool
        pass

    @staticmethod
    def hasValueChanged(
        currentValue,  # type: QualifiedValue
        previousValue,  # type: QualifiedValue
        expectedType,  # type: DataType
        deadband,  # type: float
    ):
        # type: (...) -> bool
        pass

    @staticmethod
    def isAssignable(dest, source):
        # type: (Class, Class) -> bool
        pass

    @staticmethod
    def isBoolean(clazz):
        # type: (Class) -> bool
        pass

    @staticmethod
    def isDirectlyAssignable(dest, source):
        # type: (Class, Class) -> bool
        pass

    @staticmethod
    def isFractional(clazz):
        # type: (Class) -> bool
        pass

    @staticmethod
    def isNullOrEmpty(s):
        # type: (String) -> bool
        return not s

    @staticmethod
    def isNumber(clazz):
        # type: (Class) -> bool
        pass

    @staticmethod
    def isPrimitive(clazz):
        # type: (Class) -> bool
        pass

    @staticmethod
    def isProperNumber(clazz):
        # type: (Class) -> bool
        pass

    @staticmethod
    def neq(o1, o2):
        # type: (Object, Object) -> bool
        pass

    @staticmethod
    def pyToGson(pyObject, customGson=None):
        # type: (PyObject, Optional[Gson]) -> JsonElement
        pass

    @staticmethod
    def pyToJava(pyObject):
        # type: (PyObject) -> Object
        pass

    @staticmethod
    def setArrayValue(arrayValue, newVal, pos):
        # type: (Object, Object, int) -> QualityCode
        pass

    @staticmethod
    def setClassInitializer(init):
        # type: (TypeUtilities.ClassInitializer) -> None
        pass

    @staticmethod
    def toBool(value):
        # type: (Object) -> bool
        pass

    @staticmethod
    def toByteArray(uuid):
        # type: (UUID) -> bytearray
        pass

    @staticmethod
    def toColor(color):
        # type: (Object) -> Color
        pass

    @staticmethod
    def toDataset(value):
        # type: (Object) -> Dataset
        pass

    @staticmethod
    def toDate(value):
        # type: (Object) -> Date
        pass

    @staticmethod
    def toDocument(value):
        # type: (Object) -> DocumentElement
        pass

    @staticmethod
    def toDouble(value):
        # type: (Object) -> float
        pass

    @staticmethod
    def toEnum(enumType, value):
        # type: (Class, String) -> T
        pass

    @staticmethod
    def toFloat(value):
        # type: (Object) -> float
        pass

    @staticmethod
    def toInteger(value):
        # type: (Object) -> int
        pass

    @staticmethod
    def toLong(value):
        # type: (Object) -> long
        pass

    @staticmethod
    def toNumber(value, locale=None):
        # type: (Object, Optional[Locale]) -> Number
        pass

    @staticmethod
    def toShort(value):
        # type: (Object) -> int
        pass

    @staticmethod
    def toStr(value):
        # type: (Object) -> String
        pass

    @staticmethod
    def toStringLocalized(value, locale=None):
        # type: (Object, Optional[Locale]) -> String
        pass

    @staticmethod
    def toStringOnlyNumberLocalized(value, locale):
        # type: (Object, Locale) -> String
        pass

    @staticmethod
    def toUUID(barr):
        # type: (bytearray) -> UUID
        pass

    class ClassInitializer(object):
        def createNew(self, claz):
            # type: (Class) -> Object
            pass



"""Tag Functions.

The following functions give you access to interact with Ignition Tags.
"""

from __future__ import print_function

__all__ = [
    "DEFAULT_TIMEOUT_MILLIS",
    "LEGACY_DEFAULT_TIMEOUT_MILLIS",
    "TAG_PATH",
    "browse",
    "browseHistoricalTags",
    "configure",
    "copy",
    "deleteAnnotations",
    "deleteTags",
    "exists",
    "exportTags",
    "getConfiguration",
    "importTags",
    "isOverlaysEnabled",
    "move",
    "query",
    "queryAnnotations",
    "queryTagCalculations",
    "queryTagDensity",
    "queryTagHistory",
    "readAsync",
    "readBlocking",
    "rename",
    "requestGroupExecution",
    "setOverlaysEnabled",
    "storeAnnotations",
    "storeTagHistory",
    "writeAsync",
    "writeBlocking",
]

from typing import Any, Callable, Dict, List, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.ignition.common.browsing import Results
from com.inductiveautomation.ignition.common.model.values import (
    BasicQualifiedValue,
    QualityCode,
)
from com.inductiveautomation.ignition.common.sqltags.history.annotations import (
    Annotation,
)
from java.lang import String
from java.util import Date

DEFAULT_TIMEOUT_MILLIS = 45000
LEGACY_DEFAULT_TIMEOUT_MILLIS = 45000
TAG_PATH = None  # type: Any


def browse(path, filter=None):
    # type: (String, Optional[Dict[String, Any]]) -> Results
    """Returns a list of tags found at the specified Tag path.

    The list objects are returned as dictionaries with some basic
    information about each Tag.

    Args:
        path: The path that will be browsed, typically to a folder or
            UDT instance.
        filter: A dictionary of browse filter keys.

    Returns:
        A Results object which contains a list of Tag dictionaries, one
        for each Tag found during the browse.
    """
    print(path, filter)
    return Results()


def browseHistoricalTags(
    path,  # type: String
    nameFilters=None,  # type: Optional[List[String]]
    maxSize=None,  # type: Optional[int]
    continuationPoint=None,  # type: Optional[Any]
):
    # type: (...) -> Results
    """Will browse for any historical Tags at the provided historical
    path.

    It will only browse for Tags at the path, and will not go down
    through any children. Will return with a BrowseResults object.

    Args:
        path: The Historical Tag Path to browse.
        nameFilters: A list of name filters to be applied to the result
            set. Optional.
        maxSize: The maximum size of the result set. Optional.
        continuationPoint: Sets the continuation point in order to
            continue a browse that was previously started and then
            limited. Use getContinuationPoint() on the Results object to
            get the continuation point. Optional.

    Returns:
        A Results object which contains a list of Tag dictionaries, one
        for each Tag found during the browse.
    """
    print(path, nameFilters, maxSize, continuationPoint)
    return Results()


def configure(
    basePath,  # type: String
    tags,  # type: List[Dict[String, Any]]
    collisionPolicy="o",  # type: String
):
    # type: (...) -> List[QualityCode]
    """Creates Tags from a given list of Python dictionaries or from a
    JSON source string.

    Can be used to overwrite a current Tag's configuration.

    When utilizing this function, the Tag definitions must specify the
    names of properties with their scripting/JSON name.

    Args:
        basePath: The starting point where the new Tags will be created.
            When making changes to existing tags with this function,
            you want to set the path to the parent folder of the
            existing Tag(s), not the Tag(s) themselves.
        tags: A list of Tag definitions, where each Tag definition is a
            Python dictionary. Alternately, a JSON source string may be
            passed to this parameter. When editing existing tags, it is
            generally easier to retrieve the Tag configurations with
            system.Tag.getConfiguration, modify the results of the
            getConfiguration call, and then write the new configuration
            to the parent folder of the existing Tag(s).
        collisionPolicy: The action to take when a Tag or folder with
            the same path and name is encountered. Possible values
            include:

            a - Abort and throw an exception
            o - Overwrite and replace existing Tag's configuration
            i - Ignore that item in the list
            m - Merge, modifying values that are specified in the
                definition, without impacting values that aren't defined
                in the definition. Use this when you want to apply a
                slight change to tags, without having to build a
                complete configuration object.

            Defaults to Overwrite. Optional.

    Returns:
        A List of QualityCode objects, one for each Tag in the list,
        that is representative of the result of the operation.
    """
    print(basePath, tags, collisionPolicy)
    return [QualityCode() for _ in tags]


def copy(
    tags,  # type: List[String]
    destination,  # type: String
    collisionPolicy="o",  # type: String
):
    # type: (...) -> List[QualityCode]
    """Copies tags from one folder to another.

    Multiple Tag and folder paths may be passed to a single call of this
    function. The new destination can be a separate Tag provider.

    Args:
        tags: A List of Tag paths to move.
        destination: The destination to copy the Tags to. All specified
            tags will be copied to the same destination. The destination
            Tag provider must be specified.
        collisionPolicy: The action to take when a Tag or folder with
            the same path and name is encountered. Possible values
            include: "a" Abort and throw an exception, "o" Overwrite and
            replace existing Tag's configuration, "i" Ignore that item
            in the list. Defaults to Overwrite. Optional.

    Returns:
        A List of QualityCode objects, one for each Tag in the list,
        that is representative of the result of the operation.
    """
    print(tags, destination, collisionPolicy)
    return [QualityCode() for _ in tags]


def deleteAnnotations(paths, storageIds):
    # type: (List[String], List[String]) -> List[BasicQualifiedValue]
    """Removes stored annotations from the sqlth_annotations table.

    Requires the full Tag path (including history provider) for each
    annotation, as well as each annotation's storage ID.

    The function expects two lists (PySequences) of equal length. The
    items in each list is 1-to-1, meaning the first item in the "paths"
    list relates to the first item in the "storageIds" list, the second
    item in "paths" relates to the second item in "storageIds", etc.

    Args:
        paths: A list of Tag paths with existing annotations. The paths
            are equivalent to what would be used for a Tag history
            query, and should specify the source provider as well. For
            example, "[HistoryProvider/Gateway:Provider]Path/To/Tag".
        storageIds: A sequence of storage identifiers that will be
            deleted. Storage ID values can be retrieved with
            system.tag.queryAnnotations.

    Returns:
        A list of qualified values. The quality code will indicate
        success or failure, and if successful, the storage id of the
        annotation will have been deleted.
    """
    print(paths, storageIds)
    return [BasicQualifiedValue()]


def deleteTags(tagPaths):
    # type: (List[String]) -> List[QualityCode]
    """Deletes multiple Tags or Tag Folders.

    When deleting a Tag Folder, all Tags under the folder are also
    deleted.

    Args:
        tagPaths: A List of the paths to the Tags or Tag Folders that
            are to be removed.

    Returns:
         A List of QualityCode objects, one for each Tag in the list,
         that is representative of the result of the operation.
    """
    print(tagPaths)
    return [QualityCode() for _ in tagPaths]


def exists(tagPath):
    # type: (String) -> bool
    """Checks whether or not a Tag with a given path exists.

    Args:
        tagPath: The path of the Tag to look up.

    Returns:
        True if a Tag exists for the given path, False otherwise.
    """
    print(tagPath)
    return True


def exportTags(
    filePath,  # type: String
    tagPaths,  # type: List[String]
    recursive=True,  # type: bool
    exportType="json",  # type: String
):
    # type: (...) -> None
    """Exports Tags to a file on a local file system.

    The term "local file system" refers to the scope in which the script
    was running; for example, running this script in a Gateway Timer
    script will export the file to the Gateway file system.

    Args:
        filePath: The file path that the Tags will be exported to. If
            the file does not already exist, this function will attempt
            to create it.
        tagPaths: A List of Tag paths to export. All Tag paths in the
            list must be from the same parent folder.
        recursive: Set to True to export all Tags under each Tag path,
            including Tags in child folders. Defaults to True. Optional.
        exportType: The type of file that will be exported. Set to
            "json" or "xml". Defaults to "json". Optional.
    """
    print(filePath, tagPaths, recursive, exportType)


def getConfiguration(basePath, recursive=False):
    # type: (String, bool) -> List[Dict[String, Any]]
    """Retrieves Tags from the Gateway as Python dictionaries.

    These can be edited and then saved back using system.tag.configure.

    Args:
        basePath: The starting point where the Tags will be retrieved.
            This can be a folder containing, and if recursive is True,
            then the function will attempt to retrieve all of the tags
            in the folder.
        recursive: If True, the entire Tag Tree under the specified path
            will be retrieved. Note that this will only check one level
            under the base path. True recursion would require multiple
            uses of this function at different paths. Optional.

    Returns:
         A List of Tag dictionaries. Nested Tags are placed in a list
         marked as "tags" in the dictionary.
    """
    print(basePath, recursive)
    return [{}]


def importTags(filePath, basePath, collisionPolicy="o"):
    # type: (String, String, String) -> List[QualityCode]
    """Imports a JSON Tag file at the provided path.

    Also supports XML and CSV Tag file exports from legacy systems.

    Args:
        filePath: The file path of the Tag export to import.
        basePath: The Tag path that will serve as the root node for the
            imported Tags.
        collisionPolicy: The action to take when a Tag or folder with
            the same path and name is encountered. Possible values
            include: "a" Abort and throw an exception, "o" Overwrite and
            replace existing Tag's configuration, "i" Ignore that item
            in the list. Defaults to Overwrite. Optional.

    Returns:
        A List of QualityCode objects, one for each Tag in the list,
        that is representative of the result of the operation.
    """
    print(filePath, basePath, collisionPolicy)
    return [QualityCode()]


def isOverlaysEnabled():
    # type: () -> bool
    """Returns whether or not the current client's quality overlay
    system is currently enabled.

    Returns:
         True if overlays are currently enabled.
    """
    return False


def move(tags, destination, collisionPolicy="o"):
    # type: (String, String, String) -> List[QualityCode]
    """Moves Tags or Folders to a new destination.

    The new destination can be a separate Tag provider. If interested in
    copying the tags to a new destination, instead of moving them,
    please see the system.tag.copy page.

    Args:
        tags: A List of Tag paths to move.
        destination: The destination to move the Tags to. The
            destination Tag provider must be specified: i.e.,
            [default]Folder/myTag
        collisionPolicy: The action to take when a Tag or folder with
            the same path and name is encountered. Possible values
            include: "a" Abort and throw an exception, "o" Overwrite and
            replace existing Tag's configuration, "i" Ignore that item
            in the list. Defaults to Overwrite. Optional.

    Returns:
        A List of QualityCode objects, one for each Tag in the list,
        that is representative of the result of the operation.
    """
    print(tags, destination, collisionPolicy)
    return [QualityCode() for _ in tags]


def query(
    provider=None,  # type: Optional[String]
    query=None,  # type: Optional[Dict[String, Any]]
    limit=None,  # type: Optional[int]
    continuation=None,  # type: Optional[String]
):
    # type: (...) -> Results
    """Queries a Tag Provider to produce a list of tags that meet the
    specified criteria.

    Args:
        provider: The Tag Provider to query. Optional.
        query: A JSON document that specifies the query conditions.
            Optional.
        limit: Maximum results to return. If more results are possible,
            the result will have a continuation point set. Optional.
        continuation: The Tag ID of a previously returned
            continuation point, to continue the associated query from
            that point. Optional.

    Returns:
        A dataset representing the results of the query.
    """
    print(provider, query, limit, continuation)
    return Results()


def queryAnnotations(
    paths,  # type: List[String]
    startTime=None,  # type: Optional[Date]
    endTime=None,  # type: Optional[Date]
    types=None,  # type: Optional[List[String]]
):
    # type: (...) -> List[Annotation]
    """Queries user stored annotations from the Tag history system for a
    set of paths, for a given time range.

    Args:
        paths: A list of Tag paths to query. The paths are equivalent to
            what would be used ofr a Tag history query, and should
            specify the source provider as well. For example,
            "[HistoryProvider/Gateway:Provider]Path/To/Tag".
        startTime: The start of the time range. If not defined, defaults
            to 12 hours ago. Optional.
        endTime: The end of time range. If not defined, defaults to
            "now". Optional.
        types: A list of string "types" to filter on. Types are defined
            by the annotations and various subsystems, and may vary with
            different providers. Possible annotation types are listed on
            the system.Tag.storeAnnotations page. Optional.

    Returns:
        A list of Annotation objects that match the query criteria.
    """
    print(paths, startTime, endTime, types)
    return [Annotation()]


def queryTagCalculations(
    paths,  # type: List[String]
    calculations,  # type: List[String]
    startDate=None,  # type: Optional[Date]
    endDate=None,  # type: Optional[Date]
    rangeHours=None,  # type: Optional[int]
    rangeMinutes=None,  # type: Optional[int]
    aliases=None,  # type: Optional[List[String]]
    includeBoundingValues=True,  # type: bool
    validatesSCExec=True,  # type: bool
    noInterpolation=False,  # type: bool
    ignoreBadQuality=False,  # type: bool
):
    # type: (...) -> BasicDataset
    """Queries various calculations (aggregations) for a set of tags
    over a specified range.

    Returns a dataset with a row per Tag, and a column per calculation.

    This is useful when you wish to aggregate Tag history collected over
    a period of time into a single value per aggregate. If you want
    multiple values aggregated to a single time slice (i.e., hourly
    aggregates for the same Tag over an 8 hour period) consider using
    system.Tag.queryTagHistory

    Args:
        paths: An array of Tag paths (strings) to query calculations
            for. The resulting dataset will have a row for each Tag, and
            a column for each calculation.
        calculations: An array of calculations (aggregation functions)
            to execute for each Tag. Valid values are: "Average"
            (time-weighted), "MinMax", "LastValue", "SimpleAverage",
            "Sum", "Minimum", "Maximum", "DurationOn", "DurationOff",
            "CountOn", "CountOff", "Count", "Range", "Variance",
            "StdDev", "PctGood", and "PctBad".
        startDate: The starting point for the calculation window. If
            omitted, and range is not used, 8 hours before the current
            time is used. Optional.
        endDate: The end of the calculation window. If omitted, and
            range is not used, uses the current time. Optional.
        rangeHours: Allows you to specify the query range in hours,
            instead of using start and end date. Can be positive or
            negative, and can be used in conjunction with startDate or
            endDate. Optional.
        rangeMinutes: Same as rangeHours, but in minutes. Optional.
        aliases: Aliases that will be used to override the Tag path
            names in the result dataset. Must be 1-to-1 with the Tag
            paths. If not specified, the Tag paths themselves will be
            used. Optional.
        includeBoundingValues: A boolean flag indicating that the system
            should attempt to load values before and after the query
            bounds for the purpose of interpolation. The effect depends
            on the aggregates used. The default is True. Optional.
        validatesSCExec: A boolean flag indicating whether or not data
            should be validated against the scan class execution
            records. If False, calculations may include data that is
            assumed to be good, even though the system may not have been
            running. Default is True. Optional.
        noInterpolation: A boolean flag indicating that the system
            should not attempt to interpolate values in situations where
            it normally would, such as for analog tags. Default is
            False. Optional.
        ignoreBadQuality: A boolean flag indicating that bad quality
            values should not be used in the query process. If set, any
            value with a "bad" quality will be completely ignored in
            calculations. Default is False. Optional.

    Returns:
        A dataset representing the calculations over the specified
        range.
    """
    print(
        paths,
        calculations,
        startDate,
        endDate,
        rangeHours,
        rangeMinutes,
        aliases,
        includeBoundingValues,
        validatesSCExec,
        noInterpolation,
        ignoreBadQuality,
    )
    return BasicDataset()


def queryTagDensity(paths, startDate, endDate):
    # type: (List[String], Date, Date) -> BasicDataset
    """Queries the Tag history system for information about the density
    of data.

    In other words, how much data is available for a given time
    span.

    This function is called with a list of Tag paths, and a start and
    end date. The result set is a two column dataset specifying the
    timestamp, and a relative weight. Each row is valid from the given
    time until the next row. Tags are assigned a 1 or a 0 if they are
    present or not. All values are then multiplied together to get a
    decimal based percentage for the density. Thus, for four Tag paths
    passed in, if three Tags were present during the span, the result
    would be 0.75.

    Note:
        This function relies on being able to validate the data against
        tag group execution. This function will be unable to return
        density information for tags that were stored by an Internal
        Historian Provider, as well as cases where tag group validation
        is disabled (such as by disabling Enable State Data Detection).

    Args:
        paths: An array of Tag paths (strings) to query.
        startDate: The start of the range to query.
        endDate: The end of the range to query.

    Returns:
        A two-column dataset consisting of a timestamp and a weight.
        Each row is valid until the next row.
    """
    print(paths, startDate, endDate)
    return BasicDataset()


def queryTagHistory(
    paths,  # type: List[String]
    startDate=None,  # type: Optional[Date]
    endDate=None,  # type: Optional[Date]
    returnSize=-1,  # type: int
    aggregationMode="Average",  # type: String
    returnFormat="Wide",  # type: String
    columnNames=None,  # type: Optional[List[String]]
    intervalHours=None,  # type: Optional[int]
    intervalMinutes=None,  # type: Optional[int]
    rangeHours=None,  # type: Optional[int]
    rangeMinutes=None,  # type: Optional[int]
    aggregationModes=None,  # type: Optional[List[String]]
    includeBoundingValues=None,  # type: Optional[bool]
    validateSCExec=None,  # type: Optional[bool]
    noInterpolation=None,  # type: Optional[bool]
    ignoreBadQuality=None,  # type: Optional[bool]
    timeout=None,  # type: Optional[int]
    intervalSeconds=None,  # type: Optional[int]
    rangeSeconds=None,  # type: Optional[int]
):
    # type: (...) -> BasicDataset
    """Issues a query to the Tag Historian.

    Querying Tag history involves specifying the tags and the date
    range, as well as a few optional parameters. The Tag historian will
    find the relevant history and then interpolate and aggregate it
    together into a coherent, tabular result set.

    Args:
        paths: An array of Tag paths (strings) to query. Each Tag path
            specified will be a column in the result dataset.
        startDate: The earliest value to retrieve. If omitted, 8 hours
            before current time is used. Optional.
        endDate: The latest value to retrieve. If omitted, current time
            is used. Optional.
        returnSize: The number of samples to return. -1 will return
            values as they changed, and 0 will return the "natural"
            number of values based on the logging rates of the scan
            class(es) involved. -1 is the default. Optional.
        aggregationMode: The mode to use when aggregating multiple
            samples into one time slice. Valid values are: "Average"
            (time-weighted), "MinMax", "LastValue", "SimpleAverage",
            "Sum", "Minimum", "Maximum", "DurationOn", "DurationOff",
            "CountOn", "CountOff", "Count", "Range", "Variance",
            "StdDev", "PctGood", and "PctBad". Optional.
        returnFormat: Use "Wide" to have a column per Tag queried, or
            "Tall" to have a fixed-column format. Default is "Wide".
            Optional.
        columnNames: Aliases that will be used to override the column
            names in the result dataset. Must be 1-to-1 with the Tag
            paths. If not specified, the Tag paths themselves will be
            used as column titles. Optional.
        intervalHours: Allows you to specify the window interval in
            terms of hours, as opposed to using a specific return size.
            Optional.
        intervalMinutes: Same as intervalHours, but in minutes. Can be
            used on its own, or in conjunction with intervalHours.
            Optional.
        rangeHours: Allows you to specify the query range in hours,
            instead of using start and end date. Can be positive or
            negative, and can be used in conjunction with startDate or
            endDate. Optional.
        rangeMinutes: Same as rangeHours, but in minutes. Optional.
        aggregationModes: A one-to-one list with paths specifying an
            aggregation mode per column. Optional.
        includeBoundingValues: A boolean flag indicating that the system
            should attempt to include values for the query bound times
            if possible. The default for this property depends on the
            query mode, so unless a specific behavior is desired, it is
            best to not include this parameter. Optional.
        validateSCExec: A boolean flag indicating whether or not data
            should be validated against the scan class execution
            records. If False, data will appear flat (but good quality)
            for periods of time in which the system wasn't running. If
            True, the same data would be bad quality during downtime
            periods. Optional.
        noInterpolation: A boolean flag indicating that the system
            should not attempt to interpolate values in situations where
            it normally would. This will also prevent the return of rows
            that are purely interpolated. Optional.
        ignoreBadQuality: A boolean flag indicating that bad quality
            values should not be used in the query process. If set, any
            value with a "bad" quality will be completely ignored in
            calculations and in the result set. Optional.
        timeout: Timeout in milliseconds for Client Scope. This property
            is ignored in the Gateway Scope. Optional.
        intervalSeconds: Same as intervalHours and interval Minutes, but
            in seconds. Can be used on its own, or in conjunction with
            intervalHours and intervalMinutes. Optional.
        rangeSeconds: Allows you to specify the query range in seconds,
            instead of using start and end date. Can be positive or
            negative, and can be used in conjunction with startDate or
            endDate. Optional.

    Returns:
        A dataset representing the historian values for the specified
        Tag paths. The first column will be the timestamp, and each
        column after that represents a Tag.
    """
    print(
        paths,
        startDate,
        endDate,
        returnSize,
        aggregationMode,
        returnFormat,
        columnNames,
        intervalHours,
        intervalMinutes,
        rangeHours,
        rangeMinutes,
        aggregationModes,
        includeBoundingValues,
        validateSCExec,
        noInterpolation,
        ignoreBadQuality,
        timeout,
        intervalSeconds,
        rangeSeconds,
    )
    return BasicDataset()


def readAsync(tagPaths, callback):
    # type: (List[String], Callable[..., Any]) -> None
    """Asynchronously reads the value of the Tags at the given paths.

    You must provide a python callback function that can process the
    read results.

    Args:
        tagPaths: A List of Tag paths to read from. If no property is
            specified in the path, the Value property is assumed.
        callback: A Python callback function to process the read
            results. The function definition must provide a single
            argument, which will hold a List of qualified values when
            the callback function is invoked. The qualified values will
            have three sub members: value, quality, and timestamp.
    """
    print(tagPaths, callback)


def readBlocking(tagPaths, timeout=45000):
    # type: (List[String], int) -> List[BasicQualifiedValue]
    """Reads the value of the Tags at the given paths.

    Will block until the read operation is complete or times out.

    Args:
        tagPaths: A List of Tag paths to read from. If no property is
            specified in a path, the Value property is assumed.
        timeout: How long to wait in milliseconds before the read
            operation times out. This parameter is optional, and
            defaults to 45000 milliseconds if not specified. Optional.

    Returns:
        A list of QualifiedValue objects corresponding to the Tag paths.
    """
    print(tagPaths, timeout)
    return [BasicQualifiedValue() for _ in tagPaths]


def rename(tag, newName, collisionPollicy="a"):
    # type: (String, String, String) -> QualityCode
    """Renames a single Tag or folder.

    Args:
        tag: A path to the Tag or folder to rename.
        newName: The new name for the tag or folder.
        collisionPollicy: The action to take when a Tag or folder with
            the same path and names is encountered. Possible values
            include "a" (Abort, throws an exception), "o" (Overwrite,
            completely replaces a Tag's configuration), and "i"
            (Ignore). Defaults to Abort if not specified. Optional.

    Returns:
        A QualityCode object that contains the result of the rename
        operation.
    """
    print(tag, newName, collisionPollicy)
    return QualityCode()


def requestGroupExecution(provider, tagGroup):
    # type: (String, String) -> None
    """Sends a request to the specified Tag Group to execute now.

    Args:
        provider: Name of the Tag Provider that the Tag Group is in.
        tagGroup: The name of the Tag Group to execute.
    """
    print(provider, tagGroup)


def setOverlaysEnabled(enabled):
    # type: (bool) -> None
    """Enables or disables the component quality overlay system.

    Args:
        enabled: True to turn on Tag overlays, False to turn
            them off.
    """
    print(enabled)


def storeAnnotations(
    paths,  # type: List[String]
    startTimes=None,  # type: Optional[List[Date]]
    endTimes=None,  # type: Optional[List[Date]]
    types=None,  # type: Optional[List[Annotation]]
    data=None,  # type: Optional[List[String]]
    storageIds=None,  # type: Optional[List[int]]
    deleted=None,  # type: Optional[List[bool]]
):
    # type: (...) -> List[BasicQualifiedValue]
    """Stores annotations into the Tag history system.

    Annotations are stored by the underlying historian implementations,
    so different providers may store in different ways, and some
    providers may not support annotation storage. All parameters are
    1-to-1, so all provided lists should be of the same length. If a
    particular annotation doesn't need a parameter, that element can be
    None in the list.

    Args:
        paths: A list of Tag paths to store for. The paths are
            equivalent to what would be used for a Tag history query,
            and should specify the source provider as well. For example,
            "[HistoryProvider/Gateway:Provider]Path/To/Tag". This
            parameter is required, even if storage ids are included,
            because it is used to identify the underlying storage
            provider.
        startTimes: The start times of the events. If omitted, defaults
            to the current time. Optional.
        endTimes: The end times of the event, if applicable. If omitted,
            does not store an end time for the annotation. Optional.
        types: The type id for the annotation. If not defined, "marker"
            will be used. See the Annotation Types for more details.
            Optional.
        data: Data for the annotation, such as text describing the
            meaning of the annotation. Optional.
        storageIds: If defined, the function will instead update the
            existing annotation instead of adding new ones, overriding
            existing values for the annotation with those provided by
            this function (if the corresponding delete parameter is
            True). Storage id is available on the Annotation object, and
            is returned as the result value from the storeAnnotations
            call. Optional.
        deleted: A list of booleans indicating that the individual
            annotation should be deleted. Requires storage id to be set
            as well. Optional.

    Returns:
        A list of QualifiedValues. The quality code will indicate
        success or failure. If successful,  the storage id of the
        annotation will be returned in the value.
    """
    print(paths, startTimes, endTimes, types, data, storageIds, deleted)
    return [BasicQualifiedValue() for _ in paths]


def storeTagHistory(
    historyprovider,  # type: String
    tagprovider,  # type: String
    paths,  # type: List[String]
    values,  # type: List[Any]
    qualities=None,  # type: Optional[List[int]]
    timestamps=None,  # type: Optional[List[Date]]
):
    # type: (...) -> None
    """Inserts data into the Tag history system, allowing Tag history to
    be recorded via scripting.

    The Tag paths are associated with a historical and realtime
    provider, but they do not necessarily need to exist in the realtime
    provider. This means records from non-existent (virtual) Tags can be
    stored in the Tag History system. Because of this, it is imperative
    that Tag paths passed to the function are typed precisely, otherwise
    the history will be stored at an incorrect path.

    Note that the Tag History system does cache Tag data. Thus, if this
    function is called, the Tag path and Tag id are cached until the
    history provider or gateway are restarted. This means manually
    removing the Tag from the sqlth_te table, and then calling this
    function again with the same path will not re-populate the Tag
    execution table (especially so when working purely with virtual Tag
    paths). Instead, the cache must first be cleared, and then a new
    entry will be added the next time this function is called.

    If a Tag's datatype changes after recording Tag history, this
    function will create a new entry in the sqlth_te table to reflect
    the change.

    Args:
        historyprovider: The historical provider to store to.
        tagprovider: The name of the realtime Tag provider to associate
            these tags with. The Tag provider does not need to exist,
            and the Tag paths do not need to exist in it.
        paths: A list of paths to store. The values, qualities, and
            timestamps are one-to-one with the paths. A single path may
            be present multiple times in order to store multiple values.
        values: A list of values to store.
        qualities: A list of integer quality codes corresponding to the
            values. Quality codes can be found on the Tag Quality and
            Overlays page. If omitted, GOOD quality will be used.
            Optional.
        timestamps: A list of Date timestamps corresponding to the
            values. If omitted, the current time will be used. A
            java.util.date object may be passed, so the system.date
            functions can be used to return a timestamp. Optional.
    """
    print(historyprovider, tagprovider, paths, values, qualities, timestamps)


def writeAsync(
    tagPaths,  # type: List[String]
    values,  # type: List[Any]
    callback=None,  # type: Optional[Callable[..., Any]]
):
    # type: (...) -> None
    """Asynchronously writes values to Tags a the given paths.

    You must provide a Python callback function that can process the
    write results.

    Args:
        tagPaths: A List of Tag paths to write to. If no property is
            specified in a Tag path, the Value property is assumed.
        values: The values to write to the specified paths.
        callback: A Python callback function to process the write
            results. The function definition must provide a single
            argument: a List of QualityCode objects corresponding to the
            results of the write operation. Optional.
    """
    print(tagPaths, values, callback)


def writeBlocking(
    tagPaths,  # type: List[String]
    values,  # type: List[Any]
    timeout=45000,  # type: int
):
    # type: (...) -> List[QualityCode]
    """Writes values to Tags at the given paths.

    This function will block until the write operation is complete or
    times out.

    Args:
        tagPaths: A List of Tag paths to write to. If no property is
            specified in a Tag path, the Value property is assumed.
        values: The values to write to the specified paths.
        timeout: How long to wait in milliseconds before the write
            operation times out. This parameter is optional, and
            defaults to 45,000 milliseconds if not specified. Optional.

    Returns:
        A List of QualityCode objects, one for each Tag path.
    """
    print(tagPaths, values, timeout)
    return [QualityCode() for _ in tagPaths]



from __future__ import print_function

__all__ = ["BasicUser", "ContactInfo", "PyUser", "User", "UserSourceMeta"]

from typing import Any, Iterable, List, Optional, Union

from com.inductiveautomation.ignition.common import QualifiedPath
from com.inductiveautomation.ignition.common.config import (
    BasicProperty,
    Property,
    PropertySet,
    PropertyValue,
)
from com.inductiveautomation.ignition.common.user.schedule import ScheduleAdjustment
from java.lang import Object, String
from java.util import Date


class ContactInfo(Object):
    contactType = ""  # type: String
    value = ""  # type: String

    def __init__(self, *args):
        # type: (*Any) -> None
        if len(args) == 2:
            self.contactType = args[0]
            self.value = args[1]

    def getContactType(self):
        # type: () -> String
        return self.contactType

    def getValue(self):
        # type: () -> String
        return self.value

    def setContactType(self, contactType):
        # type: (String) -> None
        self.contactType = contactType

    def setValue(self, value):
        # type: (String) -> None
        self.value = value


class User(object):
    Badge = BasicProperty("badge", str)  # type: BasicProperty
    DEFAULT_SCHEDULE_NAME = "Always"
    FirstName = BasicProperty("firstname", str)  # type: BasicProperty
    Language = BasicProperty("language", str, "en")  # type: BasicProperty
    LastName = BasicProperty("firstname", str)  # type: BasicProperty
    Notes = BasicProperty("notes", str)  # type: BasicProperty
    Password = BasicProperty("password", str)  # type: BasicProperty
    Schedule = BasicProperty("schedule", str, "Always")  # type: BasicProperty
    Username = BasicProperty("username", str)  # type: BasicProperty
    USERNAME_PATTERN = r"[\p{Alnum}][ @\w.\s\-]{1, 49}"

    def getContactInfo(self):
        # type: () -> List[ContactInfo]
        raise NotImplementedError

    def getId(self):
        # type: () -> Any
        raise NotImplementedError

    def getPath(self):
        # type: () -> QualifiedPath
        raise NotImplementedError

    def getProfileName(self):
        # type: () -> String
        raise NotImplementedError

    def getRoles(self):
        # type: () -> List[String]
        raise NotImplementedError

    def getScheduleAdjustments(self):
        # type: () -> List[ScheduleAdjustment]
        raise NotImplementedError


class BasicUser(User):
    contactInfo = None  # type: Optional[List[ContactInfo]]
    id = None  # type: Any
    profileName = None  # type: String
    roles = None  # type: List[String]
    scheduleAdjustments = None  # type: List[ScheduleAdjustment]

    def __init__(
        self,
        profileName,  # type: String
        id_,  # type: Any
        roles,  # type: List[String]
        contactInfo=None,  # type: Optional[List[ContactInfo]]
    ):
        # type: (...) -> None
        self.profileName = profileName
        self.id = id_
        self.roles = roles
        self.contactInfo = contactInfo

    def getContactInfo(self):
        # type: () -> List[ContactInfo]
        pass

    def getId(self):
        # type: () -> Any
        pass

    def getPath(self):
        # type: () -> QualifiedPath
        pass

    def getProfileName(self):
        # type: () -> String
        pass

    def getRoles(self):
        # type: () -> List[String]
        pass

    def getScheduleAdjustments(self):
        # type: () -> List[ScheduleAdjustment]
        pass


class PyUser(User):
    """A User implementation that delegates to another user object, but
    has some methods that are more scripting friendly.
    """

    _user = None

    def __init__(self, user=None):
        # type: (Optional[User]) -> None
        self._user = user

    def addContactInfo(self, *args):
        # type: (*Any) -> None
        print(self, args)

    def addRole(self, role):
        # type: (String) -> None
        print(self, role)

    def addRoles(self, roles):
        # type: (List[String]) -> None
        print(self, roles)

    def addScheduleAdjustment(self, start, end, available, note):
        # type: (Date, Date, bool, String) -> None
        print(self, start, end, available, note)

    def addScheduleAdjustments(self, scheduleAdjustments):
        # type: (List[ScheduleAdjustment]) -> None
        print(self, scheduleAdjustments)

    def contains(self, prop):
        # type: (Property) -> bool
        pass

    def get(self, propertyName):
        # type: (Union[Property, String]) -> Any
        pass

    def getContactInfo(self):
        # type: () -> List[ContactInfo]
        ci_email = ContactInfo("email", "johdoe@mycompany.com")
        ci_phone = ContactInfo("phone", "+1 5551324567")
        ci_sms = ContactInfo("sms", "+1 5557654321")
        return [ci_email, ci_phone, ci_sms]

    def getCount(self):
        # type: () -> int
        print(self)
        return 1

    def getId(self):
        # type: () -> Any
        pass

    def getOrDefault(self, prop):
        # type: (Property) -> Any
        print(self, prop)

    def getOrElse(self, property, value):
        # type: (Property, Any) -> Any
        pass

    def getPath(self):
        # type: () -> QualifiedPath
        pass

    def getProfileName(self):
        # type: () -> String
        pass

    def getProperties(self):
        # type: () -> List[Property]
        pass

    def getRoles(self):
        # type: () -> List[String]
        return ["Administrator", "Developer"]

    def getScheduleAdjustments(self):
        # type: () -> List[ScheduleAdjustment]
        pass

    def getValues(self):
        # type: () -> List[PropertyValue]
        pass

    def isExtended(self, prop):
        # type: (Property) -> bool
        pass

    def isInherited(self, prop):
        # type: (Property) -> bool
        pass

    def iterator(self):
        # type: () -> Iterable[Property]
        pass

    def merge(self, other, localOnly):
        # type: (PropertySet, bool) -> None
        pass

    def remove(self, prop):
        # type: (Property) -> None
        pass

    def removeContactInfo(self, contactType, value):
        # type: (String, String) -> None
        pass

    def removeRole(self, role):
        # type: (String) -> None
        pass

    def removeScheduleAdjustment(self, start, end, available, note):
        # type: (Date, Date, bool, String) -> None
        pass

    def set(self, *args):
        # type: (*Any) -> None
        pass

    def setContactInfo(self, contactInfo):
        # type: (List[ContactInfo]) -> None
        pass

    def setRoles(self, roles):
        # type: (List[String]) -> None
        pass

    def setScheduleAdjustments(self, scheduleAdjustments):
        # type: (List[ScheduleAdjustment]) -> None
        pass


class UserSourceMeta(Object):
    description = None  # type: String
    name = None  # type: String
    type = None  # type: String

    def __init__(self, name, description, type_):
        # type: (String, String, String) -> None
        self.name = name
        self.description = description
        self.type = type_

    def getName(self):
        # type: () -> String
        return self.name

    def getDescription(self):
        # type: () -> String
        return self.description

    def getType(self):
        # type: () -> String
        return self.type
